@@ -1,10 +1,12 @@
 #!/usr/bin/env python
 
 # This file is part of Xpra.
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2013 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+#pylint: disable=bare-except
+
 import datetime
 import subprocess
 import socket
@@ -13,14 +15,15 @@ import os.path
 import re
 import sys
 
-def bytestostr(x):
-    return str(x)
 if sys.version > '3':
     unicode = str           #@ReservedAssignment
     def bytestostr(x):
-        if type(x)==bytes:
+        if isinstance(x, bytes):
             return x.decode("latin1")
         return str(x)
+else:
+    def bytestostr(x):
+        return str(x)
 
 
 def update_properties(props, filename):
@@ -33,7 +36,7 @@ def save_properties(props, filename):
     if os.path.exists(filename):
         try:
             os.unlink(filename)
-        except:
+        except (OSError, IOError):
             print("WARNING: failed to delete %s" % filename)
     def u(v):
         try:
@@ -52,7 +55,7 @@ def save_properties(props, filename):
             s = bytestostr(value).replace("'", "\\'")
             w(name)
             w("=")
-            quote_it = type(value) not in (bool, tuple, int)
+            quote_it = not isinstance(value, (bool, tuple, int))
             if quote_it:
                 w("'")
             w(s)
@@ -70,11 +73,11 @@ def get_properties(filename):
             for line in f:
                 try:
                     s = line.decode("utf-8")
-                except:
+                except UnicodeDecodeError:
                     #str cannot be decoded!
                     s = str(line)
                 s = s.strip()
-                if len(s)==0:
+                if not s:
                     continue
                 if s[0] in ('!', '#'):
                     continue
@@ -84,6 +87,8 @@ def get_properties(filename):
                     continue
                 name = parts[0]
                 value = parts[1]
+                if not value:
+                    continue
                 if value[0]!="'" or value[-1]!="'":
                     continue
                 props[name]= value[1:-1]
@@ -116,58 +121,61 @@ def get_status_output(*args, **kwargs):
     stdout, stderr = p.communicate()
     return p.returncode, stdout, stderr
 
-def get_output_lines(commands):
-    for cmd, valid_exit_code in commands:
-        try:
-            returncode, stdout, stderr = get_status_output(cmd, stdin=None, shell=True)
-            if returncode!=valid_exit_code:
-                print("'%s' failed with return code %s" % (cmd, returncode))
-                print("stderr: %s" % stderr)
-                continue
-            if not stdout:
-                print("could not get version information")
-                continue
-            out = stdout.decode('utf-8')
-            return out.splitlines()
-        except:
-            pass
-    return  []
+def get_output_lines(cmd, valid_exit_code=0):
+    try:
+        returncode, stdout, stderr = get_status_output(cmd, stdin=None, shell=True)
+        if returncode!=valid_exit_code:
+            print("'%s' failed with return code %s" % (cmd, returncode))
+            print("stderr: %s" % stderr)
+            return ()
+        if not stdout:
+            print("could not get output from command '%s'" % (cmd,))
+            return ()
+        out = stdout.decode('utf-8')
+        return out.splitlines()
+    except Exception as e:
+        print("error running '%s': %s" % (cmd, e))
+    return  ()
 
-def get_first_line_output(commands):
-    lines = get_output_lines(commands)
-    if len(lines)>0:
+def get_first_line_output(cmd, valid_exit_code=0):
+    lines = get_output_lines(cmd, valid_exit_code)
+    if lines:
         return lines[0]
     return  ""
 
 def get_nvcc_version():
-    options = []
     for p in ("/usr/local/cuda/bin", "/opt/cuda/bin", ""):
         nvcc = os.path.join(p, "nvcc")
         if p=="" or os.path.exists(nvcc):
-            options.append(("%s --version" % (nvcc), 0))
-    lines = get_output_lines(options)
-    if len(lines)>0:
-        vline = lines[-1]
-        vpos = vline.rfind(", V")
-        if vpos>0:
-            return vline[vpos+3:]
+            cmd = "%s --version" % (nvcc)
+            lines = get_output_lines(cmd)
+            if lines:
+                vline = lines[-1]
+                vpos = vline.rfind(", V")
+                if vpos>0:
+                    return vline[vpos+3:]
     return None
 
 def get_compiler_version():
-    #FIXME: we assume we'll use GCC if it is on the path...
-    test_options = [("gcc --version", 0)]
-    if sys.platform.startswith("win"):
-        test_options.append(("cl", 0))
-        test_options.append((os.path.join(os.environ.get("VCINSTALLDIR", ""), "bin", "cl"), 0))
-    return get_first_line_output(test_options)
+    cc_version = "%s --version" % os.environ.get("CC", "gcc")
+    if sys.platform=="darwin":
+        lines = get_output_lines(cc_version)
+        for line in lines:
+            if line.startswith("Apple"):
+                return line
+        return None
+    return get_first_line_output(cc_version)
 
 def get_linker_version():
-    #FIXME: we assume we'll use GCC if it is on the path...
-    test_options = [("ld --version", 0)]
-    if sys.platform.startswith("win"):
-        test_options.append(("link", 1100))
-        test_options.append((os.path.join(os.environ.get("VCINSTALLDIR", ""), "bin", "link"), 1100))
-    return get_first_line_output(test_options)
+    if sys.platform=="darwin":
+        ld_version = "%s -v" % os.environ.get("LD", "ld")
+        lines = get_output_lines(ld_version)
+        for line in lines:
+            if line.find("using: ")>0:
+                return line.split("using: ", 1)[1]
+        return None
+    ld_version = "%s --version" % os.environ.get("LD", "ld")
+    return get_first_line_output(ld_version)
 
 
 def set_prop(props, key, value):
@@ -194,14 +202,13 @@ def get_platform_name():
     if sys.platform.find("openbsd")>=0:
         return "OpenBSD"
     if sys.platform.startswith("win"):
-        #TODO: use something a bit faster:
         try:
             o = subprocess.Popen('systeminfo', stdout=subprocess.PIPE).communicate()[0]
             try:
                 o = str(o, "latin-1")  # Python 3+
             except:
                 pass
-            return re.search("OS Name:\s*(.*)", o).group(1).strip()
+            return re.search(r"OS Name:\s*(.*)", o).group(1).strip()
         except:
             pass
         return "Microsoft Windows"
@@ -222,14 +229,24 @@ def record_build_info(is_build=True):
     global BUILD_INFO_FILE
     props = get_properties(BUILD_INFO_FILE)
     if is_build:
-        try:
-            import getpass
-            set_prop(props, "BUILT_BY", getpass.getuser())
-        except:
-            set_prop(props, "BUILT_BY", os.environ.get("USER"))
-        set_prop(props, "BUILT_ON", socket.gethostname())
-        set_prop(props, "BUILD_DATE", datetime.date.today().isoformat())
-        set_prop(props, "BUILD_TIME", datetime.datetime.now().strftime("%H:%M"))
+        source_epoch = os.environ.get("SOURCE_DATE_EPOCH")
+        if source_epoch:
+            #reproducible builds:
+            build_time = datetime.datetime.utcfromtimestamp(int(source_epoch))
+            build_date = build_time.date()
+        else:
+            #win32, macos and older build environments:
+            build_time = datetime.datetime.now()
+            build_date = datetime.date.today()
+            #also record username and hostname:
+            try:
+                import getpass
+                set_prop(props, "BUILT_BY", getpass.getuser())
+            except:
+                set_prop(props, "BUILT_BY", os.environ.get("USER"))
+            set_prop(props, "BUILT_ON", socket.gethostname())
+        set_prop(props, "BUILD_DATE", build_date.isoformat())
+        set_prop(props, "BUILD_TIME", build_time.strftime("%H:%M"))
         set_prop(props, "BUILD_MACHINE", get_machineinfo())
         set_prop(props, "BUILD_CPU", get_cpuinfo())
         set_prop(props, "BUILD_BIT", platform.architecture()[0])
@@ -267,7 +284,7 @@ def load_ignored_changed_files():
     with open("./ignored_changed_files.txt", "rU") as f:
         for line in f:
             s = line.strip()
-            if len(s)==0:
+            if not s:
                 continue
             if s[0] in ('!', '#'):
                 continue
@@ -326,7 +343,8 @@ def get_svn_props():
             #use a normalized path ("/") that does not interfere with regexp:
             norm_path = filename.replace(os.path.sep, "/")
             if norm_path==x:
-                print("'%s' matches ignore list entry: '%s' exactly, not counting it as a modified file" % (filename, x))
+                print("'%s' matches ignore list entry: '%s' exactly," % (filename, x))
+                print(" not counting it as a modified file")
                 ignore = True
                 break
             rstr = r"^%s$" % x.replace("*", ".*")
