@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 import gtk
-from xpra.x11.gtk_x11 import gdk_display_source
+from xpra.x11.gtk2 import gdk_display_source
 assert gdk_display_source
 from xpra.x11.gtk_x11.prop import prop_set
 
