@@ -1,16 +1,15 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2016 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-
-from xpra.log import Logger
-log = Logger("window")
-
 #pretend to draw the windows, but don't actually do anything
 from xpra.util import envbool
+from xpra.log import Logger
+
+log = Logger("window")
 USE_FAKE_BACKING = envbool("XPRA_USE_FAKE_BACKING", False)
 
 
@@ -30,6 +29,8 @@ class ClientWidgetBase(object):
         self._window_alpha = False
         self._client = client
         self._current_icon = None
+        self._backing = None
+        self.pixel_depth = 24
 
     def make_new_backing(self, backing_class, ww, wh, bw, bh):
         #size of the backing (same as server window source):
@@ -38,13 +39,16 @@ class ClientWidgetBase(object):
         #actual size of window (may be different when scaling):
         ww = max(1, ww)
         wh = max(1, wh)
+        if ww>=32768 or wh>=32768:
+            log.warn("Warning: invalid window dimensions %ix%i", ww, wh)
         backing = self._backing
         if backing is None:
             bc = backing_class
             if USE_FAKE_BACKING:
                 from xpra.client.fake_window_backing import FakeBacking
                 bc = FakeBacking
-            log("make_new_backing%s effective backing class=%s, server alpha=%s, window alpha=%s", (backing_class, ww, wh, ww, wh), bc, self._has_alpha, self._window_alpha)
+            log("make_new_backing%s effective backing class=%s, server alpha=%s, window alpha=%s",
+                (backing_class, ww, wh, ww, wh), bc, self._has_alpha, self._window_alpha)
             backing = bc(self._id, self._window_alpha, self.pixel_depth)
             if self._client.mmap_enabled:
                 backing.enable_mmap(self._client.mmap)
@@ -57,8 +61,8 @@ class ClientWidgetBase(object):
     def set_cursor_data(self, cursor_data):
         pass
 
-    def new_backing(self, w, h):
-        raise Exception("override me!")
+    def new_backing(self, _w, _h):
+        raise NotImplementedError("override me!")
 
     def is_OR(self):
         return False
