@@ -25,7 +25,6 @@ iconlog = Logger("icon")
 alphalog = Logger("alpha")
 
 
-REPAINT_ALL = os.environ.get("XPRA_REPAINT_ALL", "")
 SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
 PROPERTIES_DEBUG = [x.strip() for x in os.environ.get("XPRA_WINDOW_PROPERTIES_DEBUG", "").split(",")]
 AWT_DIALOG_WORKAROUND = envbool("XPRA_AWT_DIALOG_WORKAROUND", WIN32)
@@ -76,6 +75,7 @@ class ClientWindowBase(ClientWidgetBase):
         self.button_state = {}
         self.pixel_depth = pixel_depth      #0 for default
         self.window_offset = None           #actual vs reported coordinates
+        self.pending_refresh = []
 
         self.init_window(metadata)
         self.setup_window(bw, bh)
@@ -107,6 +107,7 @@ class ClientWindowBase(ClientWidgetBase):
         #initialize gravity early:
         sc = typedict(metadata.dictget("size-constraints", {}))
         self.window_gravity = OVERRIDE_GRAVITY or sc.intget("gravity", DEFAULT_GRAVITY)
+        self.set_decorated(metadata.boolget("decorations", True))
 
 
     def get_desktop_workspace(self):
@@ -251,7 +252,7 @@ class ClientWindowBase(ClientWidgetBase):
             except Exception as e:
                 log.error("Error parsing window title:")
                 log.error(" %s", e)
-                title = b""
+                title = ""
             self.set_title(title)
 
         if b"icon-title" in metadata:
@@ -339,8 +340,14 @@ class ClientWindowBase(ClientWidgetBase):
                     self.deiconify()
 
         if b"decorations" in metadata:
-            self.set_decorated(metadata.boolget("decorations"))
-            self.apply_geometry_hints(self.geometry_hints)
+            decorated = metadata.boolget("decorations", True)
+            was_decorated = self.get_decorated()
+            if WIN32 and decorated!=was_decorated:
+                log.info("decorations flag toggled, now %s, re-initializing window", decorated)
+                self.idle_add(self._client.reinit_window, self._id, self)
+            else:
+                self.set_decorated(metadata.boolget("decorations"))
+                self.apply_geometry_hints(self.geometry_hints)
 
         if b"above" in metadata:
             above = metadata.boolget("above")
@@ -649,27 +656,34 @@ class ClientWindowBase(ClientWidgetBase):
             from xpra.client.window_backing_base import fire_paint_callbacks
             fire_paint_callbacks(callbacks, -1, "no backing")
             return
-        def after_draw_refresh(success, message=""):
-            plog("after_draw_refresh(%s, %s) %sx%s at %sx%s encoding=%s, options=%s",
-                 success, message, width, height, x, y, coding, options)
-            if success<=0:
-                return
-            backing = self._backing
-            if backing and (backing.draw_needs_refresh or REPAINT_ALL):
-                if REPAINT_ALL=="1" or self._client.xscale!=1 or self._client.yscale!=1 or is_Wayland():
-                    rw, rh = self.get_size()
-                    rx, ry = 0, 0
-                else:
-                    rx, ry, rw, rh = self._client.srect(x, y, width, height)
-                #if self.window_offset:
-                #    rx -= self.window_offset[0]
-                #    ry -= self.window_offset[1]
-                self.idle_add(self.queue_draw_area, rx, ry, rw, rh)
         #only register this callback if we actually need it:
         if backing.draw_needs_refresh:
-            callbacks.append(after_draw_refresh)
+            if not backing.repaint_all:
+                self.pending_refresh.append((x, y, width, height))
+            if options.intget("flush", 0)==0:
+                callbacks.append(self.after_draw_refresh)
         backing.draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
 
+    def after_draw_refresh(self, success, message=""):
+        plog("after_draw_refresh(%s, %s) pending_refresh=%s",
+             success, message, self.pending_refresh)
+        backing = self._backing
+        if not backing:
+            return
+        if backing.repaint_all or self._client.xscale!=1 or self._client.yscale!=1 or is_Wayland():
+            #easy: just repaint the whole window:
+            rw, rh = self.get_size()
+            self.idle_add(self.queue_draw_area, 0, 0, rw, rh)
+            return
+        pr = self.pending_refresh
+        self.pending_refresh = []
+        for x, y, w, h in pr:
+            rx, ry, rw, rh = self._client.srect(x, y, w, h)
+            #if self.window_offset:
+            #    rx -= self.window_offset[0]
+            #    ry -= self.window_offset[1]
+            self.idle_add(self.queue_draw_area, rx, ry, rw, rh)
+
     def eos(self):
         """ Note: this runs from the draw thread (not UI thread) """
         backing = self._backing
