@@ -27,7 +27,9 @@ BLACKLIST = {
     "renderer" :
         [
             "Software Rasterizer",
-            #"Mesa DRI Intel(R) Ivybridge Desktop"
+            "Mesa DRI Intel(R) Ivybridge Desktop",
+            "Mesa DRI Intel(R) Haswell Mobile",
+            "Intel(R) UHD Graphics 620",
         ],
     "vendor"    : [
         #"VMware, Inc.",
