@@ -40,7 +40,7 @@ class BugReport(object):
     def setup_window(self):
         self.window = gtk.Window()
         window_defaults(self.window)
-        self.window.connect("destroy", self.close)
+        self.window.connect("delete-event", self.close)
         self.window.set_default_size(400, 300)
         self.window.set_title("Xpra Bug Report")
 
@@ -241,6 +241,7 @@ class BugReport(object):
         log("close%s", args)
         self.hide()
         self.window = None
+        return True
 
     def destroy(self, *args):
         log("destroy%s", args)
