@@ -1676,7 +1676,10 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         if is_gtk3():
             context.rectangle(0, 0, w, h)
         else:
-            context.rectangle(gdk.Rectangle(0, 0, w, h))
+            rect = gdk.Rectangle()
+            rect.width = w
+            rect.height = h
+            context.rectangle(rect)
         context.fill()
         #add spinner:
         dim = min(w/3.0, h/3.0, 100.0)
@@ -1831,8 +1834,10 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         sx, sy, sw, sh = cx(x), cy(y), cx(w), cy(h)
         packet = ["configure-window", self._id, sx, sy, sw, sh, props, self._resize_counter, state, skip_geometry]
         if self._client.window_configure_pointer:
-            #window id is redundant (now that we removed an OSX focus workaround)
-            packet.append(self._id)
+            pwid = self._id
+            if self.is_OR():
+                pwid = -1
+            packet.append(pwid)
             packet.append(self._client.get_mouse_position())
             packet.append(self._client.get_current_modifiers())
         geomlog("%s", packet)
