@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -10,6 +10,7 @@ from xpra.gtk_common.gobject_compat import import_gdk, import_glib
 from xpra.gtk_common.keymap import get_gtk_keymap
 from xpra.gtk_common.gtk_util import display_get_default, keymap_get_for_display
 from xpra.os_util import is_X11
+
 gdk = import_gdk()
 glib = import_glib()
 
@@ -22,8 +23,10 @@ class GTKKeyboardHelper(KeyboardHelper):
         #(as we may be getting dozens of such events at a time)
         self._keymap_changing = False
         self._keymap_change_handler_id = None
+        self._keymap = None
         display = display_get_default()
-        self._keymap = keymap_get_for_display(display)
+        if display:
+            self._keymap = keymap_get_for_display(display)
         self.update()
         if self._keymap:
             self._keymap_change_handler_id = self._keymap.connect("keys-changed", self.keymap_changed)
@@ -47,8 +50,9 @@ class GTKKeyboardHelper(KeyboardHelper):
                 #automatic changes not allowed!
                 log.info("ignoring keymap change: layout is locked to '%s'", self.layout_str())
                 return
-            if self.update():
-                log.info("keymap has been changed to '%s', sending updated mappings to the server", self.layout_str())
+            if self.update() and self.xkbmap_layout:
+                log.info("keymap has been changed to '%s'", self.layout_str())
+                log.info(" sending updated mappings to the server")
                 if self.xkbmap_layout:
                     self.send_layout()
                 self.send_keymap()
@@ -66,7 +70,7 @@ class GTKKeyboardHelper(KeyboardHelper):
         return old_hash!=self.hash
 
     def get_full_keymap(self):
-        return  get_gtk_keymap()
+        return get_gtk_keymap()
 
     def cleanup(self):
         KeyboardHelper.cleanup(self)
@@ -82,8 +86,8 @@ def main():
     #use gtk as display source:
     from xpra.os_util import POSIX
     if POSIX:
-        from xpra.x11.gtk_x11.gdk_display_source import init_display_source
-        init_display_source()
+        from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+        init_gdk_display_source()
     from xpra.util import print_nested_dict
     from xpra.platform import program_context
     with program_context("GTK-Keyboard", "GTK Keyboard"):
