@@ -1,23 +1,27 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2017 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
-from xpra.log import Logger
-log = Logger("keyboard")
 
 from xpra.util import nonl, csv, std, envbool, print_nested_dict, repr_ellipsized
 from xpra.os_util import POSIX, bytestostr
+from xpra.log import Logger
+
+log = Logger("keyboard")
 
 LAYOUT_GROUPS = envbool("XPRA_LAYOUT_GROUPS", True)
 
 
 class KeyboardHelper(object):
 
-    def __init__(self, net_send, keyboard_sync=True, shortcut_modifiers="auto", key_shortcuts=[], raw=False, layout="", layouts=[], variant="", variants=[], options=""):
+    def __init__(self, net_send, keyboard_sync=True,
+                 shortcut_modifiers="auto", key_shortcuts=(),
+                 raw=False, layout="", layouts=(),
+                 variant="", variants=(), options=""):
         self.reset_state()
         self.send = net_send
         self.locked = False
@@ -33,8 +37,10 @@ class KeyboardHelper(object):
         self.options = options
         #the platform class which allows us to map the keys:
         from xpra.platform.keyboard import Keyboard
-        self.keyboard = Keyboard()
-        log("KeyboardHelper(%s) keyboard=%s", (net_send, keyboard_sync, key_shortcuts, raw, layout, layouts, variant, variants, options), self.keyboard)
+        self.keyboard = Keyboard()      #pylint: disable=not-callable
+        log("KeyboardHelper(%s) keyboard=%s",
+            (net_send, keyboard_sync, key_shortcuts,
+             raw, layout, layouts, variant, variants, options), self.keyboard)
         key_repeat = self.keyboard.get_keyboard_repeat()
         if key_repeat:
             self.key_repeat_delay, self.key_repeat_interval = key_repeat
@@ -71,7 +77,7 @@ class KeyboardHelper(object):
 
     def cleanup(self):
         self.reset_state()
-        def nosend(*args):
+        def nosend(*_args):
             pass
         self.send = nosend
 
@@ -80,8 +86,7 @@ class KeyboardHelper(object):
 
 
     def parse_shortcuts(self, strs):
-        #TODO: maybe parse with re instead?
-        if len(strs)==0:
+        if not strs:
             """ if none are defined, add this as default
             it would be nicer to specify it via OptionParser in main
             but then it would always have to be there with no way of removing it
@@ -158,13 +163,13 @@ class KeyboardHelper(object):
                     args = []
                     for x in all_args.split(","):
                         x = x.strip()
-                        if len(x)==0:
+                        if not x:
                             continue
                         if (x[0]=='"' and x[-1]=='"') or (x[0]=="'" and x[-1]=="'"):
                             args.append(x[1:-1])
-                        elif x=="None":
+                        if x=="None":
                             args.append(None)
-                        elif x.find("."):
+                        if x.find("."):
                             args.append(float(x))
                         else:
                             args.append(int(x))
@@ -204,7 +209,7 @@ class KeyboardHelper(object):
                     modifiers.append(imod)
                 if not valid:
                     continue
-            #TODO: validate keyname
+            #should we be validating the keyname?
             keyname = keyspec[len(keyspec)-1]
             shortcuts.setdefault(keyname, []).append((modifiers, action, args))
             log("shortcut(%s)=%s", s, csv((modifiers, action, args)))
@@ -231,7 +236,7 @@ class KeyboardHelper(object):
         return False
 
     def _check_shortcut(self, window, key_name, modifiers, depressed, shortcut):
-        (req_mods, action, args) = shortcut
+        req_mods, action, args = shortcut
         extra_modifiers = list(modifiers)
         for rm in req_mods:
             if rm not in modifiers:
@@ -239,7 +244,7 @@ class KeyboardHelper(object):
                 return False
             try:
                 extra_modifiers.remove(rm)
-            except:
+            except ValueError:
                 pass        #same modifier listed twice?
         kmod = self.keyboard.get_keymap_modifiers()[0]
         if not kmod and self.keyboard.modifier_keys:
@@ -257,17 +262,19 @@ class KeyboardHelper(object):
             return False
         log("matched shortcut %s", shortcut)
         if not depressed:
-            """ when the key is released, just ignore it - do NOT send it to the server! """
+            #when the key is released, just ignore it - do NOT send it to the server!
             return True
         try:
             method = getattr(window, action)
-            log("key_handled_as_shortcut(%s,%s,%s,%s) found shortcut=%s, will call %s%s", window, key_name, modifiers, depressed, shortcut, method, args)
+            log("key_handled_as_shortcut(%s,%s,%s,%s) found shortcut=%s, will call %s%s",
+                window, key_name, modifiers, depressed, shortcut, method, args)
         except AttributeError as e:
             log.error("key dropped, invalid method name in shortcut %s: %s", action, e)
             return True
         try:
             method(*args)
-            log("key_handled_as_shortcut(%s,%s,%s,%s) has been handled: %s", window, key_name, modifiers, depressed, method)
+            log("key_handled_as_shortcut(%s,%s,%s,%s) has been handled: %s",
+                window, key_name, modifiers, depressed, method)
         except Exception as e:
             log.error("key_handled_as_shortcut(%s,%s,%s,%s)", window, key_name, modifiers, depressed)
             log.error(" failed to execute shortcut=%s", shortcut)
@@ -302,7 +309,7 @@ class KeyboardHelper(object):
                 if v in l or v is None:
                     return l
                 return [v]+list(l)
-            except:
+            except Exception:
                 if v is not None:
                     return [v]
                 return []
@@ -337,13 +344,29 @@ class KeyboardHelper(object):
         return _print, query, query_struct
 
     def query_xkbmap(self):
-        self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants, self.xkbmap_options = self.get_layout_spec()
-        self.xkbmap_print, self.xkbmap_query, self.xkbmap_query_struct = self.get_keymap_spec()
+        log("query_xkbmap()")
+        (
+            self.xkbmap_layout, self.xkbmap_layouts,
+            self.xkbmap_variant, self.xkbmap_variants,
+            self.xkbmap_options,
+            ) = self.get_layout_spec()
+        spec = self.get_keymap_spec()
+        self.xkbmap_print, self.xkbmap_query, self.xkbmap_query_struct = spec
+        log("query_xkbmap() get_keymap_spec()=%s", spec)
         self.xkbmap_keycodes = self.get_full_keymap()
+        log("query_xkbmap() get_full_keymap()=%s", self.xkbmap_keycodes)
         self.xkbmap_x11_keycodes = self.keyboard.get_x11_keymap()
-        self.xkbmap_mod_meanings, self.xkbmap_mod_managed, self.xkbmap_mod_pointermissing = self.keyboard.get_keymap_modifiers()
+        log("query_xkbmap() %s.get_x11_keymap()=%s", self.keyboard, self.xkbmap_x11_keycodes)
+        mods = self.keyboard.get_keymap_modifiers()
+        (
+            self.xkbmap_mod_meanings,
+            self.xkbmap_mod_managed,
+            self.xkbmap_mod_pointermissing,
+            ) = mods
+        log("query_xkbmap() %s.get_keymap_modifiers()=%s", self.keyboard, mods)
         self.update_hash()
-        log("layout=%s, layouts=%s, variant=%s, variants=%s", self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants)
+        log("layout=%s, layouts=%s, variant=%s, variants=%s",
+            self.xkbmap_layout, self.xkbmap_layouts, self.xkbmap_variant, self.xkbmap_variants)
         log("print=%s, query=%s, struct=%s", nonl(self.xkbmap_print), nonl(self.xkbmap_query), self.xkbmap_query_struct)
         log("keycodes=%s", repr_ellipsized(str(self.xkbmap_keycodes)))
         log("x11 keycodes=%s", repr_ellipsized(str(self.xkbmap_x11_keycodes)))
@@ -357,12 +380,17 @@ class KeyboardHelper(object):
             self.query_xkbmap()
 
     def layout_str(self):
-        return " / ".join([bytestostr(x) for x in (self.layout_option or self.xkbmap_layout, self.variant_option or self.xkbmap_variant) if bool(x)])
+        return " / ".join([bytestostr(x) for x in (
+            self.layout_option or self.xkbmap_layout, self.variant_option or self.xkbmap_variant) if bool(x)])
 
 
     def send_layout(self):
-        log("send_layout() layout_option=%s, xkbmap_layout=%s, variant_option=%s, xkbmap_variant=%s, xkbmap_options=%s", self.layout_option, self.xkbmap_layout, self.variant_option, self.xkbmap_variant, self.xkbmap_options)
-        self.send("layout-changed", self.layout_option or self.xkbmap_layout or "", self.variant_option or self.xkbmap_variant or "", self.xkbmap_options or "")
+        log("send_layout() layout_option=%s, xkbmap_layout=%s, variant_option=%s, xkbmap_variant=%s, xkbmap_options=%s",
+            self.layout_option, self.xkbmap_layout, self.variant_option, self.xkbmap_variant, self.xkbmap_options)
+        self.send("layout-changed",
+                  self.layout_option or self.xkbmap_layout or "",
+                  self.variant_option or self.xkbmap_variant or "",
+                  self.xkbmap_options or "")
 
     def send_keymap(self):
         log("send_keymap()")
@@ -408,17 +436,17 @@ class KeyboardHelper(object):
         kb_info = {}
         if self.xkbmap_query_struct or self.xkbmap_query:
             xkbqs = self.xkbmap_query_struct
-            if xkbqs:
+            if not xkbqs:
                 #parse query into a dict
                 from xpra.keyboard.layouts import parse_xkbmap_query
                 xkbqs = parse_xkbmap_query(self.xkbmap_query)
-            for x in ["rules", "model", "layout"]:
+            for x in ("rules", "model", "layout"):
                 v = xkbqs.get(x)
                 if v:
                     kb_info[x] = v
         if self.xkbmap_layout:
             kb_info["layout"] = self.xkbmap_layout
-        if len(kb_info)==0:
+        if not kb_info:
             log.info(" using default keyboard settings")
         else:
             log.info(" keyboard settings: %s", csv("%s=%s" % (std(k), std(v)) for k,v in kb_info.items()))
