@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,6 +11,7 @@ import os.path
 from xpra.util import envbool
 from xpra.os_util import memoryview_to_bytes
 from xpra.log import Logger, add_debug_category
+
 log = Logger("encoder", "nvfbc")
 
 USE_NVFBC_CUDA = envbool("XPRA_NVFBC_CUDA", False)
@@ -61,7 +62,8 @@ def main():
         rgb_format = image.get_pixel_format()
         try:
             img = Image.frombuffer("RGB", (w, h), pixels, "raw", rgb_format, stride, 1)
-            filename = os.path.join(os.path.expanduser(get_download_dir()), "screenshot-%s-%i.png" % (rgb_format, time.time()))
+            filename = os.path.join(os.path.expanduser(get_download_dir()),
+                                    "screenshot-%s-%i.png" % (rgb_format, time.time()))
             img.save(filename, "png")
             log.info("screenshot saved to %s", filename)
             return 0
