@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -27,35 +27,53 @@
 # super-fast connections to the X server, everything running on fast
 # computers... does being this careful to avoid sync's actually matter?)
 
+import threading
 import traceback
 
+from xpra.util import envbool
+from xpra.os_util import bytestostr
+from xpra.gtk_common.gobject_compat import import_gdk
+from xpra.log import Logger
+
 __all__ = ["XError", "trap", "xsync", "xswallow"]
 
-
-from xpra.util import envbool
 #run xpra in synchronized mode to debug X11 errors:
 XPRA_SYNCHRONIZE = envbool("XPRA_SYNCHRONIZE", False)
+XPRA_LOG_SYNC = envbool("XPRA_LOG_SYNC", False)
+VERIFY_MAIN_THREAD = envbool("XPRA_VERIFY_MAIN_THREAD", True)
 
-from xpra.log import Logger
 log = Logger("x11", "util")
 elog = Logger("x11", "util", "error")
 
-from xpra.gtk_common.gobject_compat import import_gdk
 gdk = import_gdk()
 
+
+if not VERIFY_MAIN_THREAD:
+    def verify_main_thread():
+        return
+else:
+    main_thread = threading.current_thread()
+    def verify_main_thread():
+        ct = threading.current_thread()
+        if main_thread != ct:
+            log.error("Error: invalid access from thread %s", ct)
+            traceback.print_stack()
+    verify_main_thread()
+
+
 class XError(Exception):
     def __init__(self, message):
         Exception.__init__(self)
         self.msg = message
 
     def __str__(self):
-        return "XError: %s" % str(self.msg)
+        return "XError: %s" % bytestostr(self.msg)
 
 
 xerror_to_name = None
 def get_X_error(xerror):
     global xerror_to_name
-    if type(xerror)!=int:
+    if not isinstance(xerror, int):
         return xerror
     try:
         from xpra.x11.bindings.window_bindings import constants     #@UnresolvedImport
@@ -86,14 +104,19 @@ class _ErrorManager(object):
     def __init__(self):
         self.depth = 0
 
-    def _enter(self):
+    def Xenter(self):
         assert self.depth >= 0
+        verify_main_thread()
         gdk.error_trap_push()
+        if XPRA_LOG_SYNC:
+            log("X11trap.enter at level %i", self.depth)
         self.depth += 1
 
-    def _exit(self, need_sync=True):
+    def Xexit(self, need_sync=True):
         assert self.depth >= 0
         self.depth -= 1
+        if XPRA_LOG_SYNC:
+            log("X11trap.exit at level %i, need_sync=%s", self.depth, need_sync)
         if self.depth == 0 and need_sync:
             gdk.flush()
         # This is a Xlib error constant (Success == 0)
@@ -108,17 +131,17 @@ class _ErrorManager(object):
         # that might also be raised, so suppress the XError in that case.
         value = None
         try:
-            self._enter()
+            self.Xenter()
             value = fun(*args, **kwargs)
         except Exception as e:
-            elog("_call(%s,%s,%s,%s) %s", need_sync, fun, args, kwargs, e, exc_info=True)
-            log("_call(%s,%s,%s,%s) %s", need_sync, fun, args, kwargs, e)
+            elog("_call%s", (need_sync, fun, args, kwargs), exc_info=True)
+            log("_call%s %s", (need_sync, fun, args, kwargs), e)
             try:
-                self._exit(need_sync)
+                self.Xexit(need_sync)
             except XError as ee:
                 log("XError %s detected while already in unwind; discarding", XErrorInfo(ee))
             raise
-        self._exit(need_sync)
+        self.Xexit(need_sync)
         return value
 
     def call_unsynced(self, fun, *args, **kwargs):
@@ -162,12 +185,12 @@ trap = _ErrorManager()
 class XSyncContext(object):
 
     def __enter__(self):
-        trap._enter()
+        trap.Xenter()
 
     def __exit__(self, e_typ, _e_val, _trcbak):
         #log("xsync.exit%s", (e_typ, e_val, trcbak))
         try:
-            trap._exit()
+            trap.Xexit()
         except XError as ee:
             if e_typ is None:
                 #we are not handling an exception yet, so raise this one:
@@ -182,12 +205,13 @@ xsync = XSyncContext()
 class XSwallowContext(object):
 
     def __enter__(self):
-        trap._enter()
+        trap.Xenter()
 
-    def __exit__(self, *_args):
-        #log("xswallow.exit%s", (e_typ, e_val, trcbak))
+    def __exit__(self, e_typ, e_val, trcbak):
+        if e_typ:
+            log("xswallow.exit%s", (e_typ, e_val, trcbak), exc_info=True)
         try:
-            trap._exit()
+            trap.Xexit()
         except XError as ee:
             log("XError %s detected while already in unwind; discarding", XErrorInfo(ee))
         #don't raise exceptions:
@@ -196,6 +220,24 @@ class XSwallowContext(object):
 xswallow = XSwallowContext()
 
 
+class XLogContext(object):
+
+    def __enter__(self):
+        trap.Xenter()
+
+    def __exit__(self, e_typ, e_val, trcbak):
+        if e_typ:
+            log.error("XError: %s, %s", XErrorInfo(e_typ), e_val, exc_info=True)
+        try:
+            trap.Xexit()
+        except XError as ee:
+            log("XError %s detected while already in unwind; discarding", XErrorInfo(ee))
+        #don't raise exceptions:
+        return True
+
+xlog = XLogContext()
+
+
 def verify_sync():
     if trap.depth<=0:
         log.error("Error: unmanaged X11 context")
