@@ -1,23 +1,21 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013, 2014 Antoine Martin <antoine@xpra.org>
 
 import re
 import sys
 from collections import deque
 
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, is_gtk3, import_pango, import_gobject
+from xpra.gtk_common.gtk_util import TableBuilder, label, get_xwindow, GetClipboard
+from xpra.platform.paths import get_icon
+from xpra.platform.features import CLIPBOARDS
 
 gtk = import_gtk()
 gdk = import_gdk()
 pango = import_pango()
 gobject = import_gobject()
 
-from xpra.gtk_common.gtk_util import TableBuilder, label, GetClipboard
-from xpra.platform.paths import get_icon
-from xpra.platform.features import CLIPBOARDS
-from xpra.gtk_common.gobject_compat import get_xid
-
 
 class ClipboardInstance(object):
     def __init__(self, selection, _log):
@@ -61,13 +59,13 @@ class ClipboardInstance(object):
     def log(self, msg):
         self._log(self.selection, msg)
 
-    def clear_entry(self, *args):
+    def clear_entry(self, *_args):
         self.value_entry.set_text("")
 
-    def clear_label(self, *args):
+    def clear_label(self, *_args):
         self.value_label.set_text("")
 
-    def get_targets_callback(self, c, targets, *args):
+    def get_targets_callback(self, _c, targets, *_args):
         self.log("got targets: %s" % str(targets))
         if hasattr(targets, "name"):
             self.log("target is atom: %s" % targets.name())
@@ -90,10 +88,10 @@ class ClipboardInstance(object):
             i += 1
         self.get_targets.show_all()
 
-    def do_get_targets(self, *args):
+    def do_get_targets(self, *_args):
         self.clipboard.request_targets(self.get_targets_callback, None)
 
-    def get_target_changed(self, cb):
+    def get_target_changed(self, _cb):
         target = self.get_targets.get_active_text()
         self.get_target_btn.set_sensitive(bool(target))
 
@@ -105,7 +103,7 @@ class ClipboardInstance(object):
             return val[:24]+".."
         return val
 
-    def selection_value_callback(self, cb, selection_data, *args):
+    def selection_value_callback(self, _cb, selection_data, *_args):
         #print("selection_value_callback(%s, %s, %s)" % (cb, selection_data, args))
         try:
             if selection_data.data is None:
@@ -118,52 +116,55 @@ class ClipboardInstance(object):
         except TypeError:
             try:
                 s = self.ellipsis("\\".join([str(x) for x in bytearray(selection_data.data)]))
-            except:
+            except Exception:
                 s = "!ERROR! binary data?"
         self.log("Got selection data: '%s'" % s)
         self.value_label.set_text(s)
 
-    def do_get_target(self, *args):
+    def do_get_target(self, *_args):
         self.clear_label()
         target = self.get_targets.get_active_text()
         self.log("Requesting %s" % target)
         self.clipboard.request_contents(target, self.selection_value_callback, None)
 
-    def selection_clear_cb(self, clipboard, data):
+    def selection_clear_cb(self, _clipboard, _data):
         #print("selection_clear_cb(%s, %s)", clipboard, data)
         self.log("Selection has been cleared")
 
-    def selection_get_callback(self, clipboard, selectiondata, info, *args):
-        #print("selection_get_callback(%s, %s, %s, %s) targets=%s" % (clipboard, selectiondata, info, args, selectiondata.get_targets()))
+    def selection_get_callback(self, _clipboard, selectiondata, _info, *_args):
+        #log("selection_get_callback(%s, %s, %s, %s) targets=%s",
+        #    clipboard, selectiondata, info, args, selectiondata.get_targets())
         value = self.value_entry.get_text()
         self.log("Answering selection request with value: '%s'" % self.ellipsis(value))
         selectiondata.set("STRING", 8, value)
 
-    def do_set_target(self, *args):
+    def do_set_target(self, *_args):
         target = self.set_targets.get_active_text()
         self.log("Target set to %s" % target)
         self.clipboard.set_with_data([(target, 0, 0)], self.selection_get_callback, self.selection_clear_cb)
 
-    def string_value_callback(self, cb, value, *args):
+    def string_value_callback(self, _cb, value, *_args):
         if value is None:
             value = ""
-        assert type(value)==str, "value is not a string!"
+        assert isinstance(value, str), "value is not a string!"
         self.log("Got string selection data: '%s'" % value)
         self.value_label.set_text(self.ellipsis(value))
 
-    def do_get_string(self, *args):
+    def do_get_string(self, *_args):
         #self.log("do_get_string%s on %s.%s" % (args, self, self.clipboard))
         self.clipboard.request_text(self.string_value_callback, None)
 
-    def do_set_string(self, *args):
+    def do_set_string(self, *_args):
         self.clipboard.set_text(self.ellipsis(self.value_entry.get_text()))
 
-    def owner_changed(self, cb, event):
+    def owner_changed(self, _cb, event):
         r = {}
         if not is_gtk3():
-            r = {gtk.gdk.OWNER_CHANGE_CLOSE : "close",
-                 gtk.gdk.OWNER_CHANGE_DESTROY : "destroy",
-                 gtk.gdk.OWNER_CHANGE_NEW_OWNER : "new owner"}
+            r = {
+                gtk.gdk.OWNER_CHANGE_CLOSE      : "close",
+                gtk.gdk.OWNER_CHANGE_DESTROY    : "destroy",
+                gtk.gdk.OWNER_CHANGE_NEW_OWNER  : "new owner",
+                }
         owner = self.clipboard.get_owner()
         #print("xid=%s, owner=%s" % (self.value_entry.get_window().xid, event.owner))
         weownit = (owner is not None)
@@ -222,31 +223,28 @@ class ClipboardStateInfoWindow(object):
         if icon:
             self.window.set_icon(icon)
         try:
-            self.add_event("ALL", "window=%s, xid=%#x" % (self.window, get_xid(self.window.get_window())))
-        except:
+            self.add_event("ALL", "window=%s, xid=%#x" % (self.window, get_xwindow(self.window.get_window())))
+        except Exception:
             self.add_event("ALL", "window=%s" % self.window)
 
     def add_event(self, selection, message):
         msg = message
-        if len(self.clipboards)>0:
+        if self.clipboards:
             msg = "%s : %s" % (selection, message)
         self.log.append(msg)
         self.events.set_text("\n".join(self.log))
 
-    def destroy(self, *args):
+    def destroy(self, *_args):
         gtk.main_quit()
 
 
 def main():
-    from xpra.platform import init, clean
+    from xpra.platform import program_context
     from xpra.log import enable_color
-    try:
-        init("Clipboard-Test", "Clipboard Test Tool")
+    with program_context("Clipboard-Test", "Clipboard Test Tool"):
         enable_color()
         ClipboardStateInfoWindow()
         gtk.main()
-    finally:
-        clean()
 
 
 if __name__ == "__main__":
