@@ -242,7 +242,7 @@ class StartSession(gtk.Window):
         icon = get_pixbuf("xpra")
         if icon:
             self.set_icon(icon)
-        self.connect("destroy", self.close)
+        self.connect("delete-event", self.close)
         add_close_accel(self, self.close)
 
         vbox = gtk.VBox(False, 0)
@@ -448,6 +448,7 @@ class StartSession(gtk.Window):
     def close(self, *args):
         log("close%s", args)
         self.hide()
+        return True
 
 
     def run_command(self, *_args):
