@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2012-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2013 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -13,7 +13,12 @@ This is used by the pycallgraph test wrapper.
  as only our code will be affected)
 """
 
-def make_thread(target, name, daemon=False):
-    t = Thread(target=target, name=name)
+def make_thread(target, name, daemon=False, args=()):
+    t = Thread(target=target, name=name, args=args)
     t.daemon = daemon
     return t
+
+def start_thread(target, name, daemon=False, args=()):
+    t = make_thread(target, name, daemon, args=args)
+    t.start()
+    return t
