@@ -10,7 +10,7 @@ import hashlib
 import uuid
 
 from xpra.child_reaper import getChildReaper
-from xpra.os_util import monotonic_time, bytestostr, strtobytes, POSIX, WIN32
+from xpra.os_util import monotonic_time, bytestostr, strtobytes, umask_context, POSIX, WIN32
 from xpra.util import typedict, csv, nonl, envint, envbool, engs
 from xpra.scripts.config import parse_bool
 from xpra.simple_stats import std_unit
@@ -37,34 +37,49 @@ DENY = 0
 ACCEPT = 1
 OPEN = 2
 
+def basename(filename):
+    #we can't use os.path.basename,
+    #because the remote end may have sent us a filename
+    #which is using a different pathsep
+    for sep in ("\\", "/", os.sep):
+        i = filename.rfind(sep) + 1
+        filename = filename[i:]
+    if not WIN32:
+        return filename
+    #many characters aren't allowed at all on win32:
+    tmp = ""
+    for char in filename:
+        if ord(char)<32 or char in ("<", ">", ":", "\"", "|", "?", "*"):
+            char = "_"
+        tmp += char
+    return tmp
 
 def safe_open_download_file(basefilename, mimetype):
     from xpra.platform.paths import get_download_dir
-    #make sure we use a filename that does not exist already:
     dd = os.path.expanduser(get_download_dir())
-    base = os.path.basename(basefilename)
-    wanted_filename = os.path.abspath(os.path.join(dd, base))
+    filename = os.path.abspath(os.path.join(dd, basename(basefilename)))
     ext = MIMETYPE_EXTS.get(mimetype)
-    if ext:
+    if ext and not filename.endswith("."+ext):
         #on some platforms (win32),
         #we want to force an extension
         #so that the file manager can display them properly when you double-click on them
-        if not wanted_filename.endswith("."+ext):
-            wanted_filename += "."+ext
-    filename = wanted_filename
+        filename += "."+ext
+    #make sure we use a filename that does not exist already:
+    root, ext = os.path.splitext(filename)
     base = 0
     while os.path.exists(filename):
         filelog("cannot save file as %s: file already exists", filename)
-        root, ext = os.path.splitext(wanted_filename)
         base += 1
         filename = root+("-%s" % base)+ext
+    filelog("safe_open_download_file(%s, %s) will use '%s'", basefilename, mimetype, filename)
     flags = os.O_CREAT | os.O_RDWR | os.O_EXCL
     try:
         flags |= os.O_BINARY                #@UndefinedVariable (win32 only)
     except AttributeError:
         pass
-    fd = os.open(filename, flags)
-    filelog("using filename '%s'", filename)
+    with umask_context(0o133):
+        fd = os.open(filename, flags)
+    filelog("using filename '%s', file descriptor=%s", filename, fd)
     return filename, fd
 
 
@@ -347,18 +362,27 @@ class FileTransferHandler(FileTransferAttributes):
             l.error("Error: file '%s' is too large:", basefilename)
             l.error(" %iMB, the file size limit is %iMB", filesize//1024//1024, self.file_size_limit)
             return
+        chunk_id = options.strget("file-chunk-id")
         #basefilename should be utf8:
         try:
             base = basefilename.decode("utf8")
         except:
             base = bytestostr(basefilename)
-        filename, fd = safe_open_download_file(base, mimetype)
+        try:
+            filename, fd = safe_open_download_file(base, mimetype)
+        except OSError as e:
+            filelog("cannot save file %s / %s", base, mimetype, exc_info=True)
+            filelog.error("Error: failed to save downloaded file")
+            filelog.error(" %s", e)
+            if chunk_id:
+                self.send("ack-file-chunk", chunk_id, False, "failed to create file: %s" % e, 0)
+            return
         self.file_descriptors.add(fd)
-        chunk_id = options.strget("file-chunk-id")
         if chunk_id:
             chunk_id = strtobytes(chunk_id)
-            if len(self.receive_chunks_in_progress)>=MAX_CONCURRENT_FILES:
-                self.send("ack-file-chunk", chunk_id, False, "too many file transfers in progress", 0)
+            l = len(self.receive_chunks_in_progress)
+            if l>=MAX_CONCURRENT_FILES:
+                self.send("ack-file-chunk", chunk_id, False, "too many file transfers in progress: %i" % l, 0)
                 os.close(fd)
                 return
             digest = hashlib.sha1()
@@ -699,17 +723,16 @@ class FileTransferHandler(FileTransferAttributes):
             filelog.error("Error: unknown datatype '%s'", dtype)
 
     def send_data_ask_timeout(self, send_id):
-        v = self.pending_send_data_timers.get(send_id)
-        if not v:
-            filelog.warn("Warning: send timeout, id '%s' not found!", send_id)
-            return False
+        v = self.pending_send_data.pop(send_id, None)
         try:
-            del self.pending_send_data[send_id]
             del self.pending_send_data_timers[send_id]
         except KeyError:
             pass
-        filename = v[0]
-        printit = v[4]
+        if not v:
+            filelog.warn("Warning: send timeout, id '%s' not found!", send_id)
+            return False
+        filename = v[1]
+        printit = v[5]
         filelog.warn("Warning: failed to %s file '%s',", ["send", "print"][printit], filename)
         filelog.warn(" the send approval request timed out")
         return False
@@ -761,7 +784,10 @@ class FileTransferHandler(FileTransferAttributes):
             chunk_state[-2] = 0         #timer has fired
             if chunk_state[-1]==chunk_no:
                 filelog.error("Error: chunked file transfer timed out on chunk %i", chunk_no)
-                del self.send_chunks_in_progress[chunk_id]
+                try:
+                    del self.send_chunks_in_progress[chunk_id]
+                except KeyError:
+                    pass
 
     def _process_ack_file_chunk(self, packet):
         #the other end received our send-file or send-file-chunk,
