@@ -40,25 +40,6 @@ for signame in (sig for sig in dir(signal) if sig.startswith("SIG") and not sig.
     SIGNAMES[getattr(signal, signame)] = signame
 
 
-#use cStringIO, fallback to StringIO,
-#and python3 is making life more difficult yet again:
-try:
-    from io import BytesIO as BytesIOClass              #@UnusedImport
-except ImportError:
-    try:
-        from cStringIO import StringIO as BytesIOClass  #@Reimport @UnusedImport
-    except ImportError:
-        from StringIO import StringIO as BytesIOClass   #@Reimport @UnusedImport
-assert BytesIOClass
-try:
-    from StringIO import StringIO as StringIOClass      #@UnusedImport
-except ImportError:
-    try:
-        from cStringIO import StringIO as StringIOClass #@Reimport @UnusedImport
-    except ImportError:
-        from io import StringIO as StringIOClass        #@Reimport @UnusedImport
-assert StringIOClass
-
 WIN32 = sys.platform.startswith("win")
 OSX = sys.platform.startswith("darwin")
 LINUX = sys.platform.startswith("linux")
@@ -177,7 +158,7 @@ def platform_release(release):
         try:
             import plistlib
             if PYTHON2:
-                pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist')
+                pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist') #pylint: disable=deprecated-method
             else:
                 with open(SYSTEMVERSION_PLIST, "rb") as f:
                     pl = plistlib.load(f)           #@UndefinedVariable
@@ -232,7 +213,7 @@ def get_machine_id():
     """
     v = u""
     if POSIX:
-        for filename in ["/etc/machine-id", "/var/lib/dbus/machine-id"]:
+        for filename in ("/etc/machine-id", "/var/lib/dbus/machine-id"):
             v = load_binary_file(filename)
             if v is not None:
                 break
@@ -275,28 +256,60 @@ def is_X11():
         return False
     if PYTHON2:
         return True
-    from xpra.x11.gtk3.gdk_bindings import is_X11_Display   #@UnresolvedImport
-    return is_X11_Display()
+    try:
+        from xpra.x11.gtk3.gdk_bindings import is_X11_Display   #@UnresolvedImport
+        return is_X11_Display()
+    except ImportError:
+        get_util_logger().debug("failed to load x11 bindings", exc_info=True)
+        return True
 
 def is_Wayland():
-    return os.environ.get("WAYLAND_DISPLAY") or os.environ.get("XDG_SESSION_TYPE")=="wayland"
+    backend = os.environ.get("GDK_BACKEND", "")
+    if backend=="wayland":
+        return True
+    return backend!="x11" and (
+        os.environ.get("WAYLAND_DISPLAY") or os.environ.get("XDG_SESSION_TYPE")=="wayland"
+        )
 
 
-def is_distribution_variant(variant=b"Debian", os_file="/etc/os-release"):
+def is_distribution_variant(variant=b"Debian"):
     if not POSIX:
         return False
     try:
-        if b"RedHat"==variant and get_linux_distribution()[0].startswith(variant):
+        v = load_os_release_file()
+        return any(l.find(variant)>=0 for l in v.splitlines() if l.startswith(b"NAME="))
+    except Exception:
+        pass
+    try:
+        if variant==b"RedHat" and get_linux_distribution()[0].startswith(variant):
             return True
         if get_linux_distribution()[0]==variant:
             return True
-    except:
+    except Exception:
         pass
+    return False
+
+def get_distribution_version_id():
+    if not POSIX:
+        return ""
     try:
-        v = load_binary_file(os_file)
-        return any(l.find(variant)>=0 for l in v.splitlines() if l.startswith(b"NAME="))
-    except:
-        return False
+        v = load_os_release_file()
+        for l in v.splitlines():
+            if l.startswith(b"VERSION_ID="):
+                return l.split(b"=", 1)[1]
+    except Exception:
+        pass
+    return ""
+
+os_release_file_data = False
+def load_os_release_file():
+    global os_release_file_data
+    if os_release_file_data is False:
+        try:
+            os_release_file_data = load_binary_file("/etc/os-release")
+        except (OSError, IOError):
+            os_release_file_data = None
+    return os_release_file_data
 
 def is_Ubuntu():
     return is_distribution_variant(b"Ubuntu")
@@ -332,11 +345,11 @@ def get_linux_distribution():
             p = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             out = p.communicate()[0]
             assert p.returncode==0 and out
-        except:
+        except Exception:
             try:
                 from xpra.scripts.config import python_platform
                 _linux_distribution = python_platform.linux_distribution()
-            except:
+            except Exception:
                 _linux_distribution = ("unknown", "unknown", "unknown")
         else:
             d = {}
@@ -400,10 +413,14 @@ def get_cpu_count():
             #python3:
             cpus = os.cpu_count()
         except AttributeError:
-            #python2:
-            import multiprocessing
-            cpus = multiprocessing.cpu_count()
-    except:
+            try:
+                from cpuinfo import get_cpu_info
+                cpus = int(get_cpu_info().get("count"))
+            except Exception:
+                #python2:
+                import multiprocessing
+                cpus = multiprocessing.cpu_count()
+    except Exception:
         pass
     return cpus
 
@@ -413,7 +430,7 @@ def filedata_nocrlf(filename):
     if v is None:
         get_util_logger().error("failed to load '%s'", filename)
         return None
-    return v.strip("\n\r")
+    return v.strip(b"\n\r")
 
 def load_binary_file(filename):
     if not os.path.exists(filename):
@@ -428,7 +445,23 @@ def load_binary_file(filename):
 
 #here so we can override it when needed
 def force_quit(status=1):
-    os._exit(status)
+    os._exit(status)  #pylint: disable=protected-access
+
+
+def register_SIGUSR_signals(idle_add):
+    if not os.name=="posix":
+        return
+    from xpra.util import dump_all_frames, dump_gc_frames
+    def sigusr1(*_args):
+        log = get_util_logger().info
+        log("SIGUSR1")
+        idle_add(dump_all_frames, log)
+    def sigusr2(*_args):
+        log = get_util_logger().info
+        log("SIGUSR2")
+        idle_add(dump_gc_frames, log)
+    signal.signal(signal.SIGUSR1, sigusr1)
+    signal.signal(signal.SIGUSR2, sigusr2)
 
 
 def livefds():
@@ -488,9 +521,10 @@ def close_all_fds(exceptions=()):
 
 def use_tty():
     from xpra.util import envbool
-    NOTTY = envbool("XPRA_NOTTY", False)
-    stdin = sys.stdin
-    return not os.environ.get("MSYSCON") and not NOTTY and stdin and stdin.isatty()
+    if envbool("XPRA_NOTTY", False):
+        return False
+    from xpra.platform.gui import use_stdin
+    return use_stdin()
 
 
 def shellsub(s, subs=None):
@@ -502,14 +536,14 @@ def shellsub(s, subs=None):
     return s
 
 
-def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
+def osexpand(s, actual_username="", uid=0, gid=0, subs=None):
     def expanduser(s):
         if actual_username and s.startswith("~/"):
             #replace "~/" with "~$actual_username/"
             return os.path.expanduser("~%s/%s" % (actual_username, s[2:]))
         return os.path.expanduser(s)
     from collections import OrderedDict
-    d = OrderedDict(subs)
+    d = OrderedDict(subs or {})
     d.update({
         "PID"   : os.getpid(),
         "HOME"  : expanduser("~/"),
@@ -678,8 +712,8 @@ def setbinarymode(fd):
         #turn on binary mode:
         try:
             import msvcrt
-            msvcrt.setmode(fd, os.O_BINARY)         #@UndefinedVariable
-        except:
+            msvcrt.setmode(fd, os.O_BINARY)         #@UndefinedVariable pylint: disable=no-member
+        except (OSError, IOError):
             get_util_logger().error("setting stdin to binary mode failed", exc_info=True)
 
 def find_lib_ldconfig(libname):
@@ -702,7 +736,7 @@ def find_lib_ldconfig(libname):
     p = subprocess.Popen([ldconfig, "-p"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
     data = bytestostr(p.communicate()[0])
 
-    libpath = re.search(pattern, data, re.MULTILINE)
+    libpath = re.search(pattern, data, re.MULTILINE)        #@UndefinedVariable
     if libpath:
         libpath = libpath.group(1)
     return libpath
