@@ -1,26 +1,29 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.gtk_common.gobject_compat import import_gtk, import_glib
-gtk = import_gtk()
-glib = import_glib()
-
 from xpra.util import envbool, csv
 from xpra.gtk_common.gtk_util import scaled_image
 from xpra.gtk_common.about import about
-from xpra.client.gtk_base.gtk_tray_menu_base import GTKTrayMenuBase, populate_encodingsmenu, \
-            CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_NAME_TO_LABEL, CLIPBOARD_LABELS, CLIPBOARD_DIRECTION_LABELS, CLIPBOARD_DIRECTION_NAME_TO_LABEL, \
-            SHOW_UPLOAD, SHOW_VERSION_CHECK, RUNCOMMAND_MENU, SHOW_SERVER_COMMANDS, SHOW_SHUTDOWN
+from xpra.client.gtk_base.gtk_tray_menu_base import (
+    GTKTrayMenuBase, populate_encodingsmenu,
+    CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_NAME_TO_LABEL, CLIPBOARD_LABELS,
+    CLIPBOARD_DIRECTION_LABELS, CLIPBOARD_DIRECTION_NAME_TO_LABEL,
+    SHOW_UPLOAD, SHOW_VERSION_CHECK, RUNCOMMAND_MENU, SHOW_SERVER_COMMANDS, SHOW_SHUTDOWN,
+    )
 from xpra.platform.paths import get_icon
 from xpra.platform.darwin.gui import get_OSXApplication
 from xpra.client import mixin_features
-
+from xpra.gtk_common.gobject_compat import import_gtk, import_glib
 from xpra.log import Logger
+
 log = Logger("osx", "tray", "menu")
 clipboardlog = Logger("osx", "menu", "clipboard")
 
+gtk = import_gtk()
+glib = import_glib()
+
 
 #control which menus are shown in the OSX global menu:
 SHOW_FEATURES_MENU = True
@@ -33,7 +36,8 @@ SHOW_SERVER_MENU = True
 
 SHOW_ABOUT_XPRA = True
 
-SINGLE_MENU = envbool("XPRA_OSX_SINGLE_MENU", True)
+SINGLE_MENU = envbool("XPRA_OSX_SINGLE_MENU", False)
+USE_WINDOW_MENU = envbool("XPRA_OSX_USE_WINDOW_MENU", True)
 
 
 SEPARATOR = "SEPARATOR"
@@ -64,6 +68,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
         self.hidden_window = None
         self.menus = {}             #all the "top-level" menu items we manage
         self.app_menus = {}         #the ones added to the app_menu via insert_app_menu_item (which cannot be removed!)
+        self.window_menu_item = None
         self.full = False
         self.set_client(client)
         self._clipboard_change_pending = False
@@ -150,8 +155,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
     def get_menu(self, label):
         if SINGLE_MENU:
             return self.app_menus.get(label)
-        else:
-            return self.menus.get(label)
+        return self.menus.get(label)
 
 
     def add_about(self):
@@ -173,6 +177,11 @@ class OSXMenuHelper(GTKTrayMenuBase):
         for label, submenu in reversed(menus):
             self.add_top_level_menu(label, submenu)
         self.menu_bar.show_all()
+        if USE_WINDOW_MENU:
+            self.window_menu_item = self.get_menu("Windows")
+            if self.window_menu_item:
+                macapp = get_OSXApplication()
+                macapp.set_window_menu(self.window_menu_item)
 
     def get_extra_menus(self):
         menus = []
@@ -204,9 +213,9 @@ class OSXMenuHelper(GTKTrayMenuBase):
             self.client.after_handshake(self.set_clipboard_menu, clipboard_menu)
         if mixin_features.audio and SHOW_SOUND_MENU:
             sound_menu = self.make_menu()
-            if self.client.speaker_allowed and len(self.client.speaker_codecs)>0:
+            if self.client.speaker_allowed and self.client.speaker_codecs:
                 sound_menu.add(self.make_speakermenuitem())
-            if self.client.microphone_allowed and len(self.client.microphone_codecs)>0:
+            if self.client.microphone_allowed and self.client.microphone_codecs:
                 sound_menu.add(self.make_microphonemenuitem())
             menus.append(("Sound", sound_menu))
         if mixin_features.windows and SHOW_ENCODINGS_MENU:
@@ -223,9 +232,12 @@ class OSXMenuHelper(GTKTrayMenuBase):
             menus.append(("Encoding", encodings_menu))
         if mixin_features.windows and SHOW_ACTIONS_MENU:
             actions_menu = self.make_menu()
-            actions_menu.add(self.make_refreshmenuitem())
             actions_menu.add(self.make_raisewindowsmenuitem())
-            menus.append(("Actions", actions_menu))
+            actions_menu.add(self.make_minimizewindowsmenuitem())
+            actions_menu.add(self.make_refreshmenuitem())
+            actions_menu.add(self.make_reinitmenuitem())
+            self.window_menu = actions_menu
+            menus.append(("Windows", actions_menu))
         if RUNCOMMAND_MENU or SHOW_SERVER_COMMANDS or SHOW_UPLOAD or SHOW_SHUTDOWN:
             server_menu = self.make_menu()
             if SHOW_SHUTDOWN:
@@ -246,12 +258,14 @@ class OSXMenuHelper(GTKTrayMenuBase):
 
 
     def _clipboard_direction_changed(self, item, label):
-        clipboardlog("_clipboard_direction_changed(%s, %s) clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
+        clipboardlog("_clipboard_direction_changed(%s, %s) clipboard_change_pending=%s",
+                     item, label, self._clipboard_change_pending)
         label = self.select_clipboard_menu_option(item, label, CLIPBOARD_DIRECTION_LABELS)
         self.do_clipboard_direction_changed(label or "")
 
     def _remote_clipboard_changed(self, item, label):
-        clipboardlog("_remote_clipboard_changed(%s, %s) clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
+        clipboardlog("_remote_clipboard_changed(%s, %s) clipboard_change_pending=%s",
+                     item, label, self._clipboard_change_pending)
         #ensure this is the only clipboard label selected:
         label = self.select_clipboard_menu_option(item, label, CLIPBOARD_LABELS)
         if not label:
@@ -272,18 +286,20 @@ class OSXMenuHelper(GTKTrayMenuBase):
         clipboard_item = self.checkitem(label)
         clipboard_item.set_draw_as_radio(True)
         def clipboard_option_changed(item):
-            clipboardlog("clipboard_option_changed(%s) label=%s, callback=%s clipboard_change_pending=%s", item, label, cb, self._clipboard_change_pending)
+            clipboardlog("clipboard_option_changed(%s) label=%s, callback=%s clipboard_change_pending=%s",
+                         item, label, cb, self._clipboard_change_pending)
             if cb:
                 cb(item, label)
         clipboard_item.connect("toggled", clipboard_option_changed)
         return clipboard_item
 
-    def select_clipboard_menu_option(self, item=None, label=None, labels=[]):
+    def select_clipboard_menu_option(self, item=None, label=None, labels=()):
         #ensure that only the matching menu item is selected,
         #(can be specified as a menuitem object, or using its label)
         #all the other menu items whose labels are specified will be made inactive
         #(we use a flag to prevent reentry)
-        clipboardlog("select_clipboard_menu_option(%s, %s, %s) clipboard_change_pending=%s", item, label, labels, self._clipboard_change_pending)
+        clipboardlog("select_clipboard_menu_option(%s, %s, %s) clipboard_change_pending=%s",
+                     item, label, labels, self._clipboard_change_pending)
         if self._clipboard_change_pending:
             return None
         clipboard = self.get_menu("Clipboard")
@@ -314,8 +330,8 @@ class OSXMenuHelper(GTKTrayMenuBase):
         try:
             label = CLIPBOARD_NAME_TO_LABEL.get(self.client.clipboard_helper.remote_clipboard)
             self.select_clipboard_menu_option(None, label, CLIPBOARD_LABELS)
-        except:
-            pass
+        except Exception:
+            clipboardlog("failed to select remote clipboard option in menu", exc_info=True)
         direction_label = CLIPBOARD_DIRECTION_NAME_TO_LABEL.get(self.client.client_clipboard_direction, "Disabled")
         clipboardlog("direction(%s)=%s", self.client.client_clipboard_direction, direction_label)
         self.select_clipboard_menu_option(None, direction_label, CLIPBOARD_DIRECTION_LABELS)
@@ -408,7 +424,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
                 return  None
             if size:
                 return scaled_image(pixbuf, size)
-            return  gtk.image_new_from_pixbuf(pixbuf)
-        except:
+            return gtk.image_new_from_pixbuf(pixbuf)
+        except Exception:
             log.error("get_image(%s, %s)", icon_name, size, exc_info=True)
             return  None
