@@ -1,21 +1,20 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-
-from xpra.gtk_common.gobject_compat import import_gtk
-gtk = import_gtk()
-
-from xpra.log import Logger
-log = Logger("tray", "osx")
-
 from xpra.os_util import monotonic_time
 from xpra.client.tray_base import TrayBase
 from xpra.gtk_common.gtk_util import pixbuf_new_from_data, pixbuf_new_from_file, COLORSPACE_RGB
 from xpra.platform.darwin.osx_menu import getOSXMenuHelper
 from xpra.platform.darwin.gui import set_exit_cb
 from xpra.platform.gui import ready as gui_ready
+from xpra.gtk_common.gobject_compat import import_gtk
+from xpra.log import Logger
+
+log = Logger("tray", "osx")
+
+gtk = import_gtk()
 
 #constants for attention_request:
 CRITICAL_REQUEST = 0
@@ -37,6 +36,9 @@ class OSXTray(TrayBase):
         set_exit_cb(self.quit)
 
 
+    def get_geometry(self):
+        return None
+
     def show(self):
         pass
 
@@ -53,7 +55,7 @@ class OSXTray(TrayBase):
     def ready(self):
         gui_ready()
 
-    def set_tooltip(self, text=None):
+    def set_tooltip(self, tooltip=None):
         #label cannot be set on the dock icon?
         pass
 
@@ -66,7 +68,7 @@ class OSXTray(TrayBase):
                 self.macapp.cancel_attention_request(self.last_attention_request_id)
                 self.last_attention_request_id = -1
 
-    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options={}):
+    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options=None):
         tray_icon = pixbuf_new_from_data(pixels, COLORSPACE_RGB, has_alpha, 8, w, h, rowstride)
         self.macapp.set_dock_icon_pixbuf(tray_icon)
         self.icon_timestamp = monotonic_time()
