@@ -1,19 +1,20 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.log import Logger
-log = Logger("shadow", "osx")
+import Quartz.CoreGraphics as CG    #@UnresolvedImport
 
 from xpra.util import envbool
+from xpra.os_util import memoryview_to_bytes, _buffer
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase
 from xpra.platform.darwin.keyboard_config import KeyboardConfig
 from xpra.platform.darwin.gui import get_CG_imagewrapper, take_screenshot
+from xpra.log import Logger
 
-import Quartz.CoreGraphics as CG    #@UnresolvedImport
+log = Logger("shadow", "osx")
 
 
 USE_TIMER = envbool("XPRA_OSX_SHADOW_USE_TIMER", False)
@@ -28,13 +29,13 @@ ALPHA = {
          CG.kCGImageAlphaNoneSkipFirst         : "SkipFirst",
    }
 
+BTYPES = tuple(x for x in (str, bytes, memoryview, _buffer, bytearray) if x is not None)
+
 #ensure that picture_encode can deal with pixels as NSCFData:
 def patch_picture_encode():
     from CoreFoundation import CFDataGetBytes, CFDataGetLength  #@UnresolvedImport
-    from xpra.os_util import memoryview_to_bytes, _buffer
-    btypes = [x for x in (str, bytes, memoryview, _buffer, bytearray) if x is not None]
     def pixels_to_bytes(v):
-        if type(v) in btypes:
+        if isinstance(v, BTYPES):
             return memoryview_to_bytes(v)
         l = CFDataGetLength(v)
         return CFDataGetBytes(v, (0, l), None)
@@ -94,7 +95,8 @@ class ShadowServer(GTKShadowServerBase):
 
     def make_tray_widget(self):
         from xpra.client.gtk_base.statusicon_tray import GTKStatusIconTray
-        return GTKStatusIconTray(self, 0, self.tray, "Xpra Shadow Server", None, None, self.tray_click_callback, mouseover_cb=None, exit_cb=self.tray_exit_callback)
+        return GTKStatusIconTray(self, 0, self.tray, "Xpra Shadow Server", None, None,
+                                 self.tray_click_callback, mouseover_cb=None, exit_cb=self.tray_exit_callback)
 
 
     def setup_capture(self):
@@ -115,7 +117,7 @@ class ShadowServer(GTKShadowServerBase):
 
     def do_screen_refresh(self, rlist):
         #TODO: improve damage method to handle lists directly:
-        from xpra.server.window.region import rectangle     #@UnresolvedImport
+        from xpra.rectangle import rectangle     #@UnresolvedImport
         model_rects = {}
         for model in self._id_to_window.values():
             model_rects[model] = rectangle(*model.geometry)
@@ -124,7 +126,9 @@ class ShadowServer(GTKShadowServerBase):
                 mrect = rect.intersection(x, y, w, h)
                 #log("screen refresh intersection of %s and %24s: %s", model, (x, y, w, h), mrect)
                 if mrect:
-                    self._damage(model, mrect.x-rect.x, mrect.y-rect.y, mrect.width, mrect.height)
+                    rx = mrect.x-rect.x
+                    ry = mrect.y-rect.y
+                    self.refresh_window_area(model, rx, ry, mrect.width, mrect.height, {"damage" : True})
 
     def start_refresh(self, wid):
         #don't use the timer, get damage notifications:
@@ -138,9 +142,8 @@ class ShadowServer(GTKShadowServerBase):
             if err==0:
                 self.refresh_registered = True
                 return
-            else:
-                log.warn("Warning: CGRegisterScreenRefreshCallback failed with error %i", err)
-                log.warn(" using fallback timer method")
+            log.warn("Warning: CGRegisterScreenRefreshCallback failed with error %i", err)
+            log.warn(" using fallback timer method")
         GTKShadowServerBase.start_refresh(self, wid)
 
     def stop_refresh(self, wid):
@@ -240,7 +243,7 @@ def main():
         filename = "./Capture-%s-%s.png" % ((x, y, w, h),tstr,)
         i.save(filename, "png")
         print("saved to %s" % (filename,))
-        
+
 
 if __name__ == "__main__":
     main()
