@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,8 +10,9 @@ import socket
 import errno
 import stat
 
-from xpra.os_util import get_util_logger, osexpand
+from xpra.os_util import get_util_logger, osexpand, umask_context
 from xpra.platform.dotxpra_common import PREFIX, LIVE, DEAD, UNKNOWN, INACCESSIBLE
+from xpra.platform import platform_import
 
 
 def norm_makepath(dirpath, name):
@@ -19,16 +20,17 @@ def norm_makepath(dirpath, name):
         name = name[1:]
     return os.path.join(dirpath, PREFIX + name)
 
-def debug(msg, *args):
+def debug(msg, *args, **kwargs):
     log = get_util_logger()
-    log(msg, *args)
+    log(msg, *args, **kwargs)
 
 
 class DotXpra(object):
-    def __init__(self, sockdir=None, sockdirs=[], actual_username="", uid=0, gid=0):
+    def __init__(self, sockdir=None, sockdirs=None, actual_username="", uid=0, gid=0):
         self.uid = uid or os.getuid()
         self.gid = gid or os.getgid()
         self.username = actual_username
+        sockdirs = sockdirs or []
         if not sockdir:
             if sockdirs:
                 sockdir = sockdirs[0]
@@ -51,16 +53,13 @@ class DotXpra(object):
                 uid = self.uid
             if gid is None:
                 gid = self.gid
-            try:
-                umask = os.umask(0)
+            with umask_context(0):
                 os.mkdir(d, mode)
-            finally:
-                os.umask(umask)
             if uid!=os.getuid() or gid!=os.getgid():
                 os.lchown(d, uid, gid)
 
     def socket_expand(self, path):
-        return self.osexpand(path, uid=self.uid, gid=self.gid)
+        return osexpand(path, self.username, uid=self.uid, gid=self.gid)
 
     def norm_socket_paths(self, local_display_name):
         return [norm_makepath(x, local_display_name) for x in self._sockdirs]
@@ -75,27 +74,32 @@ class DotXpra(object):
 
     def get_server_state(self, sockpath, timeout=5):
         if not os.path.exists(sockpath):
-            return self.DEAD
+            return DotXpra.DEAD
         sock = socket.socket(socket.AF_UNIX)
         sock.settimeout(timeout)
         try:
             sock.connect(sockpath)
-            return self.LIVE
+            return DotXpra.LIVE
         except socket.error as e:
-            debug("get_server_state: connect(%s)=%s", sockpath, e)
+            debug("get_server_state: connect(%s)=%s (timeout=%s)", sockpath, e, timeout)
             err = e.args[0]
             if err==errno.EACCES:
-                return self.INACCESSIBLE
+                return DotXpra.INACCESSIBLE
             if err==errno.ECONNREFUSED:
                 #could be the server is starting up
-                return self.UNKNOWN
-            if err in (errno.EWOULDBLOCK, errno.ENOENT):
-                return self.DEAD
+                debug("ECONNREFUSED")
+                return DotXpra.UNKNOWN
+            if err==errno.EWOULDBLOCK:
+                debug("EWOULDBLOCK")
+                return DotXpra.DEAD
+            if err==errno.ENOENT:
+                debug("ENOENT")
+                return DotXpra.DEAD
             return self.UNKNOWN
         finally:
             try:
                 sock.close()
-            except:
+            except IOError:
                 debug("%s.close()", sock, exc_info=True)
 
 
@@ -105,7 +109,8 @@ class DotXpra(object):
     #this is imported by winswitch, so we can't change the method signature
     def sockets(self, check_uid=0, matching_state=None):
         #flatten the dictionnary into a list:
-        return list(set((v[0], v[1]) for details_values in self.socket_details(check_uid, matching_state).values() for v in details_values))
+        return list(set((v[0], v[1]) for details_values in
+                        self.socket_details(check_uid, matching_state).values() for v in details_values))
 
     def socket_paths(self, check_uid=0, matching_state=None, matching_display=None):
         paths = []
@@ -114,6 +119,44 @@ class DotXpra(object):
                 paths.append(socket_path)
         return paths
 
+    def get_display_state(self, display):
+        dirs = []
+        if self._sockdir!="undefined":
+            dirs.append(self._sockdir)
+        dirs += [x for x in self._sockdirs if x not in dirs]
+        debug("get_display_state(%s) sockdir=%s, sockdirs=%s, testing=%s",
+              display, self._sockdir, self._sockdirs, dirs)
+        seen = set()
+        state = None
+        for d in dirs:
+            if not d or not os.path.exists(d):
+                debug("get_display_state: '%s' path does not exist", d)
+                continue
+            real_dir = os.path.realpath(d)
+            if real_dir in seen:
+                continue
+            seen.add(real_dir)
+            #ie: "~/.xpra/HOSTNAME-"
+            base = os.path.join(d, PREFIX)
+            potential_sockets = glob.glob(base + display.lstrip(":"))
+            for sockpath in sorted(potential_sockets):
+                try:
+                    s = os.stat(sockpath)
+                except OSError as e:
+                    debug("get_display_state: '%s' path cannot be accessed: %s", sockpath, e)
+                    #socket cannot be accessed
+                    continue
+                if stat.S_ISSOCK(s.st_mode):
+                    local_display = ":"+sockpath[len(base):]
+                    if local_display!=display:
+                        debug("get_display_state: '%s' display does not match (%s vs %s)",
+                              sockpath, local_display, display)
+                        continue
+                    state = self.get_server_state(sockpath)
+                    if state not in (self.DEAD, self.INACCESSIBLE):
+                        return state
+        return state or self.DEAD
+
     #find the matching sockets, and return:
     #(state, local_display, sockpath) for each socket directory we probe
     def socket_details(self, check_uid=0, matching_state=None, matching_display=None):
@@ -123,7 +166,8 @@ class DotXpra(object):
         if self._sockdir!="undefined":
             dirs.append(self._sockdir)
         dirs += [x for x in self._sockdirs if x not in dirs]
-        debug("socket_details%s sockdir=%s, sockdirs=%s, testing=%s", (check_uid, matching_state, matching_display), self._sockdir, self._sockdirs, dirs)
+        debug("socket_details%s sockdir=%s, sockdirs=%s, testing=%s",
+              (check_uid, matching_state, matching_display), self._sockdir, self._sockdirs, dirs)
         seen = set()
         for d in dirs:
             if not d or not os.path.exists(d):
@@ -135,7 +179,10 @@ class DotXpra(object):
             seen.add(real_dir)
             #ie: "~/.xpra/HOSTNAME-"
             base = os.path.join(d, PREFIX)
-            potential_sockets = glob.glob(base + "*")
+            if matching_display:
+                potential_sockets = glob.glob(base + matching_display.lstrip(":"))
+            else:
+                potential_sockets = glob.glob(base + "*")
             results = []
             for sockpath in sorted(potential_sockets):
                 try:
@@ -150,14 +197,11 @@ class DotXpra(object):
                             #socket uid does not match
                             debug("socket_details: '%s' uid does not match (%s vs %s)", sockpath, s.st_uid, check_uid)
                             continue
-                    local_display = ":"+sockpath[len(base):]
-                    if matching_display and local_display!=matching_display:
-                        debug("socket_details: '%s' display does not match (%s vs %s)", sockpath, local_display, matching_display)
-                        continue
                     state = self.get_server_state(sockpath)
                     if matching_state and state!=matching_state:
                         debug("socket_details: '%s' state does not match (%s vs %s)", sockpath, state, matching_state)
                         continue
+                    local_display = ":"+sockpath[len(base):]
                     results.append((state, local_display, sockpath))
             if results:
                 sd[d] = results
@@ -165,7 +209,6 @@ class DotXpra(object):
 
 
 #win32 re-defines DotXpra for namedpipes:
-from xpra.platform import platform_import
 platform_import(globals(), "dotxpra", False,
                 "DotXpra",
                 "norm_makepath")
