@@ -1,26 +1,30 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # Platform-specific code for Win32.
+#pylint: disable=bare-except
 
 import errno
 import os.path
 import sys
 import ctypes
-
-from xpra.util import envbool
-from xpra.os_util import PYTHON2
 from ctypes import WINFUNCTYPE, WinDLL, POINTER, byref, c_int
 from ctypes.wintypes import BOOL, HANDLE, DWORD, LPWSTR, LPCWSTR, LPVOID, POINT, WORD, SMALL_RECT
 
+from xpra.util import envbool
+from xpra.os_util import PYTHON2, PYTHON3
 from xpra.platform.win32 import constants as win32con
-from xpra.platform.win32.common import SetConsoleTitleA, GetConsoleScreenBufferInfo, MessageBoxA, GetLastError, kernel32
+from xpra.platform.win32.common import (
+    GetStdHandle,
+    SetConsoleTitleA, GetConsoleScreenBufferInfo,
+    MessageBoxA, GetLastError, kernel32,
+    )
 
 
-GetStdHandle = WINFUNCTYPE(HANDLE, DWORD)(("GetStdHandle", kernel32))
+STD_INPUT_HANDLE = DWORD(-10)
 STD_OUTPUT_HANDLE = DWORD(-11)
 STD_ERROR_HANDLE = DWORD(-12)
 GetFileType = WINFUNCTYPE(DWORD, DWORD)(("GetFileType", kernel32))
@@ -65,7 +69,7 @@ def is_wine():
             import _winreg as winreg
         except ImportError:
             import winreg   #@UnresolvedImport @Reimport
-        hKey = winreg.OpenKey(win32con.HKEY_LOCAL_MACHINE, r"Software\\Wine")
+        hKey = winreg.OpenKey(win32con.HKEY_LOCAL_MACHINE, r"Software\\Wine")   #@UndefinedVariable
         return hKey is not None
     except:
         #no wine key, assume not present and wait for input
@@ -85,8 +89,17 @@ def set_prgname(name):
     except:
         pass
 
+def not_a_console(handle):
+    if handle == INVALID_HANDLE_VALUE or handle is None:
+        return True
+    return ((GetFileType(handle) & ~FILE_TYPE_REMOTE) != FILE_TYPE_CHAR
+            or GetConsoleMode(handle, byref(DWORD())) == 0)
 
 def fix_unicode_out():
+    if PYTHON3:
+        _unicode = str
+    else:
+        _unicode = unicode  #@UndefinedVariable
     #code found here:
     #http://stackoverflow.com/a/3259271/428751
     import codecs
@@ -120,12 +133,6 @@ def fix_unicode_out():
         # <http://msdn.microsoft.com/en-us/library/ms683167(VS.85).aspx>
         # BOOL WINAPI GetConsoleMode(HANDLE hConsole, LPDWORD lpMode);
 
-        def not_a_console(handle):
-            if handle == INVALID_HANDLE_VALUE or handle is None:
-                return True
-            return ((GetFileType(handle) & ~FILE_TYPE_REMOTE) != FILE_TYPE_CHAR
-                    or GetConsoleMode(handle, byref(DWORD())) == 0)
-
         old_stdout_fileno = None
         old_stderr_fileno = None
         if hasattr(sys.stdout, 'fileno'):
@@ -184,11 +191,11 @@ def fix_unicode_out():
                 def write(self, text):
                     try:
                         if self._hConsole is None:
-                            if isinstance(text, unicode):
+                            if isinstance(text, _unicode):
                                 text = text.encode('utf-8')
                             self._stream.write(text)
                         else:
-                            if not isinstance(text, unicode):
+                            if not isinstance(text, _unicode):
                                 text = str(text).decode('utf-8')
                             remaining = len(text)
                             while remaining:
@@ -244,30 +251,14 @@ class CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
         ("dwMaximumWindowSize", COORD),
         ]
 
-_wait_for_input = False
-def set_wait_for_input():
-    global _wait_for_input
-    wfi = os.environ.get("XPRA_WAIT_FOR_INPUT")
-    if wfi is not None:
-        _wait_for_input = wfi!="0"
-        return
-    if is_wine():
-        #don't wait for input when running under wine
-        #(which usually does not popup a new shell window)
-        _wait_for_input = False
-        return
-    if os.environ.get("MSYSCON"):
-        #msys environment doesn't popup a new shell window
-        _wait_for_input = False
-        return
+def get_console_position(handle):
     try:
-        handle = GetStdHandle(STD_OUTPUT_HANDLE)
         #handle.SetConsoleTextAttribute(FOREGROUND_BLUE)
         csbi = CONSOLE_SCREEN_BUFFER_INFO()
         GetConsoleScreenBufferInfo(handle, byref(csbi))
         cpos = csbi.dwCursorPosition
         #wait for input if this is a brand new console:
-        _wait_for_input = cpos.X==0 and cpos.Y==0
+        return cpos.X and cpos.Y
     except:
         e = sys.exc_info()[1]
         code = -1
@@ -285,6 +276,49 @@ def set_wait_for_input():
                 print("error accessing console %s: %s" % (errno.errorcode.get(e.errno, e.errno), e))
             except:
                 print("error accessing console: %s" % e)
+        return -1, -1
+
+_wait_for_input = False
+def set_wait_for_input():
+    global _wait_for_input
+    _wait_for_input = should_wait_for_input()
+
+def should_wait_for_input():
+    wfi = os.environ.get("XPRA_WAIT_FOR_INPUT")
+    if wfi is not None:
+        return wfi!="0"
+    if is_wine():
+        #don't wait for input when running under wine
+        #(which usually does not popup a new shell window)
+        return False
+    if os.environ.get("TERM", "")=="xterm":
+        #msys, cygwin and git bash environments don't popup a new shell window
+        #and they all set TERM=xterm
+        return False
+    handle = GetStdHandle(STD_OUTPUT_HANDLE)
+    if not_a_console(handle):
+        return False
+    #wait for input if this is a brand new console:
+    return get_console_position(handle)==(0, 0)
+
+
+def setup_console_event_listener(handler, enable):
+    try:
+        from xpra.platform.win32.common import SetConsoleCtrlHandler, ConsoleCtrlHandler
+        from xpra.log import Logger
+        log = Logger("win32")
+        log("calling SetConsoleCtrlHandler(%s, %s)", handler, enable)
+        ctypes_handler = ConsoleCtrlHandler(handler)
+        result = SetConsoleCtrlHandler(ctypes_handler, enable)
+        log("SetConsoleCtrlHandler(%s, %s)=%s", handler, enable, result)
+        if result==0:
+            log.error("Error: could not %s console control handler:", "set" if enable else "unset")
+            log.error(" SetConsoleCtrlHandler: %r", ctypes.GetLastError())
+            return False
+        return True
+    except Exception as e:
+        log.error("SetConsoleCtrlHandler error: %s", e)
+        return False
 
 def do_init():
     if FIX_UNICODE_OUT:
@@ -294,12 +328,15 @@ def do_init():
         set_wait_for_input()
         return
     if envbool("XPRA_LOG_TO_FILE", True):
-        from xpra.platform import get_prgname
-        LOG_FILENAME = (get_prgname() or "Xpra")+".log"
-        from xpra.platform.win32.paths import _get_data_dir
-        d = _get_data_dir()
-        log_file = os.path.join(d, LOG_FILENAME)
-        sys.stdout = open(log_file, "a")
+        log_filename = os.environ.get("XPRA_LOG_FILENAME")
+        if not log_filename:
+            from xpra.platform.win32.paths import _get_data_dir
+            from xpra.platform import get_prgname
+            data_dir = _get_data_dir(False)
+            if not os.path.exists(data_dir):
+                os.mkdir(data_dir)
+            log_filename = os.path.join(data_dir, (get_prgname() or "Xpra")+".log")
+        sys.stdout = open(log_filename, "a")
         sys.stderr = sys.stdout
 
 
@@ -332,10 +369,10 @@ def do_clean():
         print("\nPress Enter to close")
         try:
             sys.stdout.flush()
-        except:
+        except IOError:
             pass
         try:
             sys.stderr.flush()
-        except:
+        except IOError:
             pass
         sys.stdin.readline()
