@@ -302,6 +302,9 @@ GetKeyState.restype = SHORT
 GetKeyboardLayout = user32.GetKeyboardLayout
 GetKeyboardLayoutList = user32.GetKeyboardLayoutList
 GetKeyboardLayoutList.argtypes = [c_int, POINTER(HANDLE*32)]
+GetKeyboardLayoutName = user32.GetKeyboardLayoutNameA
+GetKeyboardLayoutName.restype = BOOL
+GetKeyboardLayoutName.argtypes = [LPSTR]
 SystemParametersInfoA = user32.SystemParametersInfoA
 EnumWindows = user32.EnumWindows
 EnumWindowsProc = WINFUNCTYPE(BOOL, HWND, LPARAM)
