@@ -1,19 +1,22 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import ctypes
 from ctypes.wintypes import HANDLE
 
-from xpra.platform.win32.common import GetKeyState, GetKeyboardLayoutList, GetKeyboardLayout, GetIntSystemParametersInfo
+from xpra.platform.win32.common import GetKeyState, GetKeyboardLayoutList, GetKeyboardLayout, GetIntSystemParametersInfo, GetKeyboardLayoutName
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.keyboard_base import KeyboardBase
-from xpra.keyboard.layouts import WIN32_LAYOUTS
+from xpra.keyboard.layouts import WIN32_LAYOUTS, WIN32_KEYBOARDS
 from xpra.gtk_common.keymap import KEY_TRANSLATIONS
 from xpra.util import csv, envint, envbool
+from xpra.os_util import bytestostr
 from xpra.log import Logger
+from ctypes import create_string_buffer
+
 log = Logger("keyboard")
 
 
@@ -45,6 +48,7 @@ class Keyboard(KeyboardBase):
         self.num_lock_modifier = None
         self.altgr_modifier = None
         self.delayed_event = None
+        self.last_layout_message = None
         #workaround for "period" vs "KP_Decimal" with gtk2 (see ticket #586):
         #translate "period" with keyval=46 and keycode=110 to KP_Decimal:
         KEY_TRANSLATIONS[("period",     46,     110)]   = "KP_Decimal"
@@ -109,7 +113,39 @@ class Keyboard(KeyboardBase):
         return  {}, [], ["lock"]
 
     def get_layout_spec(self):
+        KL_NAMELENGTH = 9
+        name_buf = create_string_buffer(KL_NAMELENGTH)
         layout = None
+        if GetKeyboardLayoutName(name_buf):
+            log("get_layout_spec() GetKeyboardLayoutName()=%s", bytestostr(name_buf.value))
+            try:
+                #win32 API returns a hex string
+                ival = int(name_buf.value, 16)
+            except ValueError:
+                log.warn("Warning: failed to parse keyboard layout code '%s'", bytestostr(name_buf.value))
+            else:
+                found = False
+                for val in (ival, ival & 0xffff):
+                    kbdef = WIN32_KEYBOARDS.get(val)
+                    log("get_layout_spec() WIN32_KEYBOARDS[%#x]=%s", val, kbdef)
+                    if kbdef:
+                        found = True
+                        layout, descr = kbdef
+                        if layout=="??":
+                            log.warn("Warning: the X11 codename for %#x is not known", val)
+                            log.warn(" only identified as '%s'", descr)
+                            log.warn(" please file a bug report")
+                            layout = None
+                            continue
+                        if self.last_layout_message!=layout:
+                            log.info("keyboard layout code %#x", ival)
+                            log.info("identified as '%s' : %s", descr, layout)
+                            self.last_layout_message = layout
+                        break
+                if not found and self.last_layout_message!=layout:
+                    log.warn("Warning: unknown keyboard layout %#x", val)
+                    log.warn(" please file a bug report")
+                    self.last_layout_message = layout
         layouts = []
         variant = None
         variants = None
@@ -132,12 +168,15 @@ class Keyboard(KeyboardBase):
             log("GetKeyboardLayout(0)=%#x", hkl)
             kbid = hkl & 0xffff
             if kbid in WIN32_LAYOUTS:
-                code, _, _, _, layout, variants = WIN32_LAYOUTS.get(kbid)
-                log("found keyboard layout '%s' with variants=%s, code '%s' for kbid=%i (%#x)", layout, variants, code, kbid, hkl)
-            if not layout:
+                code, _, _, _, layout0, variants = WIN32_LAYOUTS.get(kbid)
+                log("found keyboard layout '%s' with variants=%s, code '%s' for kbid=%i (%#x)", layout0, variants, code, kbid, hkl)
+            if not layout0:
                 log("unknown keyboard layout for kbid: %i (%#x)", kbid, hkl)
-            else:
-                layouts.append(layout)
+            elif layout0 not in layouts:
+                layouts.append(layout0)
+            #only override "layout" if unset:
+            if not layout and layout0:
+                layout = layout0
         except Exception as e:
             log.error("Error: failed to detect keyboard layout:")
             log.error(" %s", e)
