@@ -74,7 +74,7 @@ def get_native_system_tray_classes():
 
 def get_wm_name():
     wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "")
-    if os.environ.get("XDG_SESSION_TYPE")=="wayland":
+    if os.environ.get("XDG_SESSION_TYPE")=="wayland" or os.environ.get("GDK_BACKEND")=="wayland":
         wm_name = "wayland"
     elif is_X11():
         try:
@@ -208,12 +208,13 @@ def _get_randr_dpi():
         from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
         with xlog:
             randr_bindings = RandRBindings()
-            wmm, hmm = randr_bindings.get_screen_size_mm()
-            w, h =  randr_bindings.get_screen_size()
-            dpix = iround(w * 25.4 / wmm)
-            dpiy = iround(h * 25.4 / hmm)
-            screenlog("xdpi=%s, ydpi=%s - size-mm=%ix%i, size=%ix%i", dpix, dpiy, wmm, hmm, w, h)
-            return dpix, dpiy
+            if randr_bindings.has_randr():
+                wmm, hmm = randr_bindings.get_screen_size_mm()
+                w, h =  randr_bindings.get_screen_size()
+                dpix = iround(w * 25.4 / wmm)
+                dpiy = iround(h * 25.4 / hmm)
+                screenlog("xdpi=%s, ydpi=%s - size-mm=%ix%i, size=%ix%i", dpix, dpiy, wmm, hmm, w, h)
+                return dpix, dpiy
     return -1, -1
 
 def get_xdpi():
@@ -370,7 +371,8 @@ def get_vrefresh():
         try:
             from xpra.x11.bindings.randr_bindings import RandRBindings      #@UnresolvedImport
             randr = RandRBindings()
-            v = randr.get_vrefresh()
+            if randr.has_randr():
+                v = randr.get_vrefresh()
         except Exception as e:
             log("get_vrefresh()", exc_info=True)
             log.warn("Warning: failed to query the display vertical refresh rate:")
