@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2018-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,10 +12,6 @@ from xpra.server.auth.sys_auth_base import SysAuthenticatorBase, init, log, pars
 from xpra.log import enable_debug_for, is_debug_enabled
 assert init and log #tests will disable logging from here
 
-def init(opts):
-    pass
-
-
 LDAP_CACERTFILE = os.environ.get("XPRA_LDAP_CACERTFILE")
 
 
@@ -110,7 +106,7 @@ def main(argv):
     from xpra.platform import program_context
     with program_context("LDAP3-Password-Auth", "LDAP3-Password-Authentication"):
         for x in list(argv):
-            if x=="-v" or x=="--verbose":
+            if x in ("-v", "--verbose"):
                 enable_debug_for("auth")
                 argv.remove(x)
         if len(argv) not in (3,4,5,6):
