@@ -1,12 +1,12 @@
 # This file is part of Xpra.
-# Copyright (C) 2014-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 
 from xpra.net.digest import get_salt, choose_digest
 
-def init(opts):
+def init(_opts):
     pass
 
 
