@@ -1,15 +1,18 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from ctypes import windll, byref, addressof, POINTER, FormatError, GetLastError
+from ctypes.wintypes import LPCWSTR, DWORD, HANDLE, BOOL
 
 from xpra.server.auth.sys_auth_base import SysAuthenticator, init, log
+from xpra.util import envbool
+from xpra.os_util import bytestostr
+from xpra.platform.win32.common import CloseHandle
 assert init and log #tests will disable logging from here
 
-from xpra.platform.win32.common import CloseHandle
-from ctypes import windll, byref, POINTER, FormatError, GetLastError
-from ctypes.wintypes import LPCWSTR, DWORD, HANDLE, BOOL
+LOG_CREDENTIALS = envbool("XPRA_LOG_CREDENTIALS", False)
 
 MAX_COMPUTERNAME_LENGTH = 15
 LOGON32_LOGON_NETWORK_CLEARTEXT = 8
@@ -36,16 +39,20 @@ class Authenticator(SysAuthenticator):
     def check(self, password):
         token = HANDLE()
         domain = '' #os.environ.get('COMPUTERNAME')
-        status = LogonUser(self.username, domain, password,
+        password_str = bytestostr(password)
+        if LOG_CREDENTIALS:
+            log("LogonUser(%s, %s, %s, CLEARTEXT, DEFAULT, %#x)",
+                self.username, domain, password_str, addressof(token))
+        status = LogonUser(self.username, domain, password_str,
                      LOGON32_LOGON_NETWORK_CLEARTEXT,
                      LOGON32_PROVIDER_DEFAULT,
                      byref(token))
-        error = GetLastError()
+        log("LogonUser(..)=%#x", status)
         if status:
             CloseHandle(token)
             return True
         log.error("Error: win32 authentication failed:")
-        log.error(" %s", FormatError(error))
+        log.error(" %s", FormatError(GetLastError()))
         return False
 
     def __repr__(self):
@@ -53,7 +60,6 @@ class Authenticator(SysAuthenticator):
 
 
 def main():
-    import sys
     from xpra.platform import program_context
     from xpra.log import enable_color
     with program_context("Auth-Test", "Auth-Test"):
@@ -72,9 +78,8 @@ def main():
         if a.check(password):
             log.info("authentication succeeded")
             return 0
-        else:
-            log.error("authentication failed")
-            return 1
+        log.error("authentication failed")
+        return 1
 
 if __name__ == "__main__":
     import sys
