@@ -1,34 +1,41 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-
+import sys
 import os.path
 
+from xpra.util import flatten_dict, envbool
+from xpra.os_util import monotonic_time, register_SIGUSR_signals, WIN32
+from xpra.gtk_common.gobject_compat import (
+    import_gdk, import_glib, is_gtk3,
+    register_os_signals,
+    )
+from xpra.gtk_common.quit import (
+    gtk_main_quit_really,
+    gtk_main_quit_on_fatal_exceptions_enable,
+    gtk_main_quit_on_fatal_exceptions_disable,
+    )
+from xpra.server import server_features
+from xpra.server.server_base import ServerBase
+from xpra.gtk_common.gtk_util import (
+    get_gtk_version_info, gtk_main, display_get_default, get_root_size,
+    keymap_get_for_display, icon_theme_get_default, ICON_SIZE_BUTTON,
+    )
 from xpra.log import Logger
+
+UI_THREAD_WATCHER = envbool("XPRA_UI_THREAD_WATCHER")
+
 log = Logger("server", "gtk")
 screenlog = Logger("server", "screen")
 cursorlog = Logger("server", "cursor")
 notifylog = Logger("notify")
 
-from xpra.util import flatten_dict
-from xpra.os_util import monotonic_time, BytesIOClass
-from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_glib, import_gobject, is_gtk3
-from xpra.gtk_common.quit import (gtk_main_quit_really,
-                           gtk_main_quit_on_fatal_exceptions_enable,
-                           gtk_main_quit_on_fatal_exceptions_disable)
-from xpra.server.server_base import ServerBase
-from xpra.gtk_common.gtk_util import get_gtk_version_info, gtk_main, display_get_default, get_root_size, keymap_get_for_display
-
 glib = import_glib()
-glib.threads_init()
-gobject = import_gobject()
-gobject.threads_init()
-gtk = import_gtk()
 gdk = import_gdk()
 
 
@@ -46,6 +53,7 @@ class GTKServerBase(ServerBase):
         self.timeout_add = glib.timeout_add
         self.source_remove = glib.source_remove
         self.cursor_suspended = False
+        self.ui_watcher = None
         ServerBase.__init__(self)
 
     def watch_keymap_changes(self):
@@ -54,6 +62,10 @@ class GTKServerBase(ServerBase):
         keymap = keymap_get_for_display(display)
         keymap.connect("keys-changed", self._keys_changed)
 
+    def install_signal_handlers(self, callback):
+        register_os_signals(callback)
+        register_SIGUSR_signals(glib.idle_add)
+
     def signal_quit(self, signum, frame=None):
         gtk_main_quit_on_fatal_exceptions_disable()
         super(GTKServerBase, self).signal_quit(signum, frame)
@@ -64,7 +76,49 @@ class GTKServerBase(ServerBase):
         gtk_main_quit_really()
         log("do_quit: gtk_main_quit_really done")
 
+    def do_cleanup(self):
+        ServerBase.do_cleanup(self)
+        self.close_gtk_display()
+        uiw = self.ui_watcher
+        if uiw:
+            uiw.stop()
+
+    def close_gtk_display(self):
+        # Close our display(s) first, so the server dying won't kill us.
+        # (if gtk has been loaded)
+        gdk_mod = sys.modules.get("gtk.gdk") or sys.modules.get("gi.repository.Gdk")
+        #bug 2328: python3 shadow server segfault on Ubuntu 16.04
+        from xpra.os_util import getUbuntuVersion, is_Ubuntu, PYTHON2
+        safe_close = PYTHON2 or self.session_type!="shadow" or not is_Ubuntu() or getUbuntuVersion()>(16, 4)
+        if WIN32:
+            safe_close = False
+        close = envbool("XPRA_CLOSE_GTK_DISPLAY", safe_close)
+        log("close_gtk_display() safe_close=%s, close=%s, gdk_mod=%s",
+            safe_close, close, gdk_mod)
+        if close and gdk_mod:
+            if is_gtk3():
+                displays = gdk.DisplayManager.get().list_displays()
+            else:
+                displays = gdk.display_manager_get().list_displays()
+            log("close_gtk_display() displays=%s", displays)
+            for d in displays:
+                log("close_gtk_display() closing %s", d)
+                d.close()
+
+
     def do_run(self):
+        if UI_THREAD_WATCHER:
+            from xpra.platform.ui_thread_watcher import get_UI_watcher
+            self.ui_watcher = get_UI_watcher(glib.timeout_add, glib.source_remove)
+            self.ui_watcher.start()
+        if server_features.windows:
+            display = display_get_default()
+            i=0
+            while i<display.get_n_screens():
+                screen = display.get_screen(i)
+                screen.connect("size-changed", self._screen_size_changed)
+                screen.connect("monitors-changed", self._monitors_changed)
+                i += 1
         gtk_main_quit_on_fatal_exceptions_enable()
         log("do_run() calling %s", gtk_main)
         gtk_main()
@@ -103,7 +157,7 @@ class GTKServerBase(ServerBase):
         if self.cursor_suspended:
             return
         self.cursor_suspended = True
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if ss:
             ss.cancel_cursor_timer()
             ss.send_empty_cursor()
@@ -113,7 +167,7 @@ class GTKServerBase(ServerBase):
         if not self.cursor_suspended:
             return
         self.cursor_suspended = False
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if ss:
             ss.send_cursor()
 
@@ -121,7 +175,7 @@ class GTKServerBase(ServerBase):
     def send_initial_cursors(self, ss, _sharing=False):
         #cursors: get sizes and send:
         display = display_get_default()
-        self.cursor_sizes = display.get_default_cursor_size(), display.get_maximal_cursor_size()
+        self.cursor_sizes = int(display.get_default_cursor_size()), display.get_maximal_cursor_size()
         cursorlog("send_initial_cursors() cursor_sizes=%s", self.cursor_sizes)
         ss.send_cursor()
 
@@ -172,7 +226,7 @@ class GTKServerBase(ServerBase):
                 continue
             for display in screen_sizes:
                 #avoid error with old/broken clients:
-                if not display or type(display) not in (list, tuple):
+                if not display or not isinstance(display, (list, tuple)):
                     continue
                 #display: [':0.0', 2560, 1600, 677, 423, [['DFP2', 0, 0, 2560, 1600, 646, 406]], 0, 0, 2560, 1574]
                 if len(display)>=10:
@@ -244,10 +298,10 @@ class GTKServerBase(ServerBase):
                 w, h = img.size
         else:
             #try to find it in the theme:
-            theme = gtk.icon_theme_get_default()
+            theme = icon_theme_get_default()
             if theme:
                 try:
-                    icon = theme.load_icon(icon_string, gtk.ICON_SIZE_BUTTON, 0)
+                    icon = theme.load_icon(icon_string, ICON_SIZE_BUTTON, 0)
                 except Exception as e:
                     notifylog("failed to load icon '%s' from default theme: %s", icon_string, e)
                 else:
@@ -263,7 +317,8 @@ class GTKServerBase(ServerBase):
             if w>256 or h>256:
                 img = img.resize((256, 256), Image.ANTIALIAS)
                 w = h = 256
-            buf = BytesIOClass()
+            from io import BytesIO
+            buf = BytesIO()
             img.save(buf, "PNG")
             cpixels = buf.getvalue()
             buf.close()
