@@ -186,14 +186,15 @@ class ServerBaseControlCommands(StubServerMixin):
         if client_uuids_str=="UI":
             sources = [ss for ss in self._server_sources.values() if ss.ui_client]
             client_uuids = [ss.uuid for ss in sources]
-            notfound = []
+            notfound = ()
         elif client_uuids_str=="*":
             sources = self._server_sources.values()
             client_uuids = [ss.uuid for ss in sources]
         else:
             client_uuids = client_uuids_str.split(",")
             sources = [ss for ss in self._server_sources.values() if ss.uuid in client_uuids]
-            notfound = [x for x in client_uuids if x not in [ss.uuid for ss in sources]]
+            uuids = tuple(ss.uuid for ss in sources)
+            notfound = any(x for x in client_uuids if x not in uuids)
             if notfound:
                 log.warn("client connection not found for uuid(s): %s", notfound)
         return sources
@@ -204,7 +205,8 @@ class ServerBaseControlCommands(StubServerMixin):
             log(msg)
             return msg
         sources = self._control_get_sources(client_uuids)
-        log("control_command_send_notification(%i, %s, %s, %s) will send to sources %s (matching %s)", nid, title, message, client_uuids, sources, client_uuids)
+        log("control_command_send_notification(%i, %s, %s, %s) will send to sources %s (matching %s)",
+            nid, title, message, client_uuids, sources, client_uuids)
         count = 0
         for source in sources:
             if source.notify(0, nid, "control channel", 0, "", title, message, [], {}, 10, ""):
@@ -271,7 +273,8 @@ class ServerBaseControlCommands(StubServerMixin):
         file_size = len(data)
         file_size_MB = file_size//1024//1024
         if file_size_MB>self.file_transfer.file_size_limit:
-            raise ControlError("file '%s' is too large: %iMB (limit is %iMB)" % (filename, file_size_MB, self.file_transfer.file_size_limit))
+            raise ControlError("file '%s' is too large: %iMB (limit is %iMB)" % (
+                filename, file_size_MB, self.file_transfer.file_size_limit))
         #send it to each client:
         for ss in sources:
             #ie: ServerSource.file_transfer (found in FileTransferAttributes)
@@ -332,7 +335,7 @@ class ServerBaseControlCommands(StubServerMixin):
     def all_send_client_command(self, *client_command):
         """ forwards the command to all clients """
         for source in tuple(self._server_sources.values()):
-            """ forwards to *the* client, if there is *one* """
+            # forwards to *the* client, if there is *one*
             if client_command[0] not in source.control_commands:
                 log.info("client command '%s' not forwarded to client %s (not supported)", client_command, source)
             else:
@@ -552,7 +555,8 @@ class ServerBaseControlCommands(StubServerMixin):
             elif press in ("0", "unpress"):
                 press = False
             else:
-                raise ControlError("if present, the press argument must be one of: %s", ("1", "press", "0", "unpress"))
+                raise ControlError("if present, the press argument must be one of: %s" %
+                                   csv(("1", "press", "0", "unpress")))
         self.fake_key(keycode, press)
 
     def control_command_sound_output(self, *args):
@@ -564,11 +568,11 @@ class ServerBaseControlCommands(StubServerMixin):
     def control_command_workspace(self, wid, workspace):
         window = self._id_to_window.get(wid)
         if not window:
-            raise ControlError("window %s does not exist", wid)
+            raise ControlError("window %s does not exist" % wid)
         if "workspace" not in window.get_property_names():
-            raise ControlError("cannot set workspace on window %s", window)
+            raise ControlError("cannot set workspace on window %s" % window)
         if workspace<0:
-            raise ControlError("invalid workspace value: %s", workspace)
+            raise ControlError("invalid workspace value: %s" % workspace)
         window.set_property("workspace", workspace)
         return "window %s moved to workspace %s" % (wid, workspace)
 
