@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -10,27 +10,16 @@ from time import sleep
 from threading import Event
 from collections import deque
 
-from xpra.log import Logger
-log = Logger("server")
-elog = Logger("encoding")
-keylog = Logger("keyboard")
-mouselog = Logger("mouse")
-timeoutlog = Logger("timeout")
-proxylog = Logger("proxy")
-statslog = Logger("stats")
-notifylog = Logger("notify")
-netlog = Logger("network")
-bandwidthlog = Logger("bandwidth")
-
-
 from xpra.make_thread import start_thread
 from xpra.os_util import Queue, monotonic_time
 from xpra.util import merge_dicts, flatten_dict, notypedict, envbool, envint, typedict, AtomicInteger
+from xpra.net.compression import compressed_wrapper, Compressed
 from xpra.server.source.source_stats import GlobalPerformanceStatistics
-
 from xpra.server.source.clientinfo_mixin import ClientInfoMixin
-CC_BASES = [ClientInfoMixin]
 from xpra.server import server_features
+from xpra.log import Logger
+
+CC_BASES = [ClientInfoMixin]
 #TODO: notifications mixin
 if server_features.clipboard:
     from xpra.server.source.clipboard_connection import ClipboardConnection
@@ -73,14 +62,21 @@ from xpra.server.source.idle_mixin import IdleMixin
 CC_BASES.append(IdleMixin)
 CC_BASES = tuple(CC_BASES)
 ClientConnectionClass = type('ClientConnectionClass', CC_BASES, {})
+
+log = Logger("server")
+elog = Logger("encoding")
+proxylog = Logger("proxy")
+notifylog = Logger("notify")
+bandwidthlog = Logger("bandwidth")
+
 log("ClientConnectionClass%s", CC_BASES)
 
 
 BANDWIDTH_DETECTION = envbool("XPRA_BANDWIDTH_DETECTION", True)
 MIN_BANDWIDTH = envint("XPRA_MIN_BANDWIDTH", 5*1024*1024)
 AUTO_BANDWIDTH_PCT = envint("XPRA_AUTO_BANDWIDTH_PCT", 80)
-assert AUTO_BANDWIDTH_PCT>1 and AUTO_BANDWIDTH_PCT<=100, "invalid value for XPRA_AUTO_BANDWIDTH_PCT: %i" % AUTO_BANDWIDTH_PCT
-NOYIELD = not envbool("XPRA_YIELD", False)
+assert 1<AUTO_BANDWIDTH_PCT<=100, "invalid value for XPRA_AUTO_BANDWIDTH_PCT: %i" % AUTO_BANDWIDTH_PCT
+YIELD = envbool("XPRA_YIELD", False)
 
 counter = AtomicInteger()
 
@@ -125,15 +121,20 @@ class ClientConnection(ClientConnectionClass):
             except Exception as e:
                 raise Exception("failed to initialize %s: %s" % (bc, e))
 
-        self.packet_queue = deque()                 #holds actual packets ready for sending (already encoded)
-                                                    #these packets are picked off by the "protocol" via 'next_packet()'
-                                                    #format: packet, wid, pixels, start_send_cb, end_send_cb
-                                                    #(only packet is required - the rest can be 0/None for clipboard packets)
+        for c in CC_BASES:
+            c.init_state(self)
+
+        #holds actual packets ready for sending (already encoded)
+        #these packets are picked off by the "protocol" via 'next_packet()'
+        #format: packet, wid, pixels, start_send_cb, end_send_cb
+        #(only packet is required - the rest can be 0/None for clipboard packets)
+        self.packet_queue = deque()
         # the encode work queue is used by mixins that need to encode data before sending it,
         # ie: encodings and clipboard
-        self.encode_work_queue = None               #this queue will hold functions to call to compress data (pixels, clipboard)
-                                                    #items placed in this queue are picked off by the "encode" thread,
-                                                    #the functions should add the packets they generate to the 'packet_queue'
+        #this queue will hold functions to call to compress data (pixels, clipboard)
+        #items placed in this queue are picked off by the "encode" thread,
+        #the functions should add the packets they generate to the 'packet_queue'
+        self.encode_work_queue = None
         self.encode_thread = None
         self.ordinary_packets = []
         self.socket_dir = socket_dir
@@ -154,6 +155,7 @@ class ClientConnection(ClientConnectionClass):
         self.init()
 
         # ready for processing:
+        self.queue_encode = self.start_queue_encode
         protocol.set_packet_source(self.next_packet)
 
 
@@ -169,6 +171,7 @@ class ClientConnection(ClientConnectionClass):
         self.share = False
         self.lock = False
         self.control_commands = ()
+        self.xdg_menu_update = False
         self.bandwidth_limit = self.server_bandwidth_limit
         self.soft_bandwidth_limit = self.bandwidth_limit
         self.bandwidth_warnings = True
@@ -182,8 +185,6 @@ class ClientConnection(ClientConnectionClass):
         self.wants_display = True
         self.wants_events = False
         self.wants_default_cursor = False
-        for c in CC_BASES:
-            c.init_state(self)
 
 
     def is_closed(self):
@@ -197,6 +198,17 @@ class ClientConnection(ClientConnectionClass):
         self.protocol = None
 
 
+    def compressed_wrapper(self, datatype, data, min_saving=128):
+        if self.zlib or self.lz4 or self.lzo:
+            cw = compressed_wrapper(datatype, data, zlib=self.zlib, lz4=self.lz4, lzo=self.lzo, can_inline=False)
+            if len(cw)+min_saving<=len(data):
+                #the compressed version is smaller, use it:
+                return cw
+            #skip compressed version: fall through
+        #we can't compress, so at least avoid warnings in the protocol layer:
+        return Compressed(datatype, data, can_inline=True)
+
+
     def update_bandwidth_limits(self):
         if not self.bandwidth_detection:
             return
@@ -217,7 +229,8 @@ class ClientConnection(ClientConnectionClass):
         if bandwidth_limit>0:
             bandwidth_limit = max(MIN_BANDWIDTH, bandwidth_limit)
         self.soft_bandwidth_limit = bandwidth_limit
-        bandwidthlog("update_bandwidth_limits() bandwidth_limit=%s, soft bandwidth limit=%s", self.bandwidth_limit, bandwidth_limit)
+        bandwidthlog("update_bandwidth_limits() bandwidth_limit=%s, soft bandwidth limit=%s",
+                     self.bandwidth_limit, bandwidth_limit)
         #figure out how to distribute the bandwidth amongst the windows,
         #we use the window size,
         #(we should use the number of bytes actually sent: framerate, compression, etc..)
@@ -261,15 +274,18 @@ class ClientConnection(ClientConnectionClass):
         self.share = c.boolget("share")
         self.lock = c.boolget("lock")
         self.control_commands = c.strlistget("control_commands")
+        self.xdg_menu_update = c.boolget("xdg-menu-update")
         bandwidth_limit = c.intget("bandwidth-limit", 0)
         server_bandwidth_limit = self.server_bandwidth_limit
         if self.server_bandwidth_limit is None:
             server_bandwidth_limit = self.get_socket_bandwidth_limit() or bandwidth_limit
         self.bandwidth_limit = min(server_bandwidth_limit, bandwidth_limit)
-        self.bandwidth_detection = c.boolget("bandwidth-detection", True)
-        cd = typedict(c.dictget("connection-data"))
-        self.jitter = cd.intget("jitter", 0)
-        bandwidthlog("server bandwidth-limit=%s, client bandwidth-limit=%s, value=%s, detection=%s", server_bandwidth_limit, bandwidth_limit, self.bandwidth_limit, self.bandwidth_detection)
+        if self.bandwidth_detection:
+            self.bandwidth_detection = c.boolget("bandwidth-detection", True)
+        self.client_connection_data = c.dictget("connection-data", {})
+        self.jitter = typedict(self.client_connection_data).intget("jitter", 0)
+        bandwidthlog("server bandwidth-limit=%s, client bandwidth-limit=%s, value=%s, detection=%s",
+                     server_bandwidth_limit, bandwidth_limit, self.bandwidth_limit, self.bandwidth_detection)
 
         cinfo = self.get_connect_info()
         for i,ci in enumerate(cinfo):
@@ -279,7 +295,6 @@ class ClientConnection(ClientConnectionClass):
             msg = version_compat_check(self.proxy_version)
             if msg:
                 proxylog.warn("Warning: proxy version may not be compatible: %s", msg)
-        self.update_connection_data(c.dictget("connection-data"))
         if getattr(self, "mmap_size", 0)>0:
             log("mmap enabled, ignoring bandwidth-limit")
             self.bandwidth_limit = 0
@@ -311,13 +326,14 @@ class ClientConnection(ClientConnectionClass):
     #
     # The encode thread loop management:
     #
-    def queue_encode(self, item):
+    def start_queue_encode(self, item):
         #start the encode work queue:
-        self.encode_work_queue = Queue()            #holds functions to call to compress data (pixels, clipboard)
-                                                    #items placed in this queue are picked off by the "encode" thread,
-                                                    #the functions should add the packets they generate to the 'packet_queue'
+        #holds functions to call to compress data (pixels, clipboard)
+        #items placed in this queue are picked off by the "encode" thread,
+        #the functions should add the packets they generate to the 'packet_queue'
+        self.encode_work_queue = Queue()
         self.queue_encode = self.encode_work_queue.put
-        self.encode_work_queue.put(item)
+        self.queue_encode(item)
         self.encode_thread = start_thread(self.encode_loop, "encode")
 
     def encode_queue_size(self):
@@ -334,7 +350,8 @@ class ClientConnection(ClientConnectionClass):
         self.statistics.compression_work_qsizes.append((monotonic_time(), self.encode_queue_size()))
         self.queue_encode(fn_and_args)
 
-    def queue_packet(self, packet, wid=0, pixels=0, start_send_cb=None, end_send_cb=None, fail_cb=None, wait_for_more=False):
+    def queue_packet(self, packet, wid=0, pixels=0,
+                     start_send_cb=None, end_send_cb=None, fail_cb=None, wait_for_more=False):
         """
             Add a new 'draw' packet to the 'packet_queue'.
             Note: this code runs in the non-ui thread
@@ -342,7 +359,9 @@ class ClientConnection(ClientConnectionClass):
         now = monotonic_time()
         self.statistics.packet_qsizes.append((now, len(self.packet_queue)))
         if wid>0:
-            self.statistics.damage_packet_qpixels.append((now, wid, sum(x[2] for x in tuple(self.packet_queue) if x[1]==wid)))
+            self.statistics.damage_packet_qpixels.append(
+                (now, wid, sum(x[2] for x in tuple(self.packet_queue) if x[1]==wid))
+                )
         self.packet_queue.append((packet, wid, pixels, start_send_cb, end_send_cb, fail_cb, wait_for_more))
         p = self.protocol
         if p:
@@ -374,19 +393,21 @@ class ClientConnection(ClientConnectionClass):
                 else:
                     log.error("Error during encoding:", exc_info=True)
                 del e
-            NOYIELD or sleep(0)
+            if YIELD:
+                sleep(0)
 
     ######################################################################
     # network:
     def next_packet(self):
         """ Called by protocol.py when it is ready to send the next packet """
-        packet, start_send_cb, end_send_cb, fail_cb, synchronous, have_more, will_have_more = None, None, None, None, True, False, False
+        packet, start_send_cb, end_send_cb, fail_cb = None, None, None, None
+        synchronous, have_more, will_have_more = True, False, False
         if not self.is_closed():
-            if len(self.ordinary_packets)>0:
+            if self.ordinary_packets:
                 packet, synchronous, fail_cb, will_have_more = self.ordinary_packets.pop(0)
-            elif len(self.packet_queue)>0:
+            elif self.packet_queue:
                 packet, _, _, start_send_cb, end_send_cb, fail_cb, will_have_more = self.packet_queue.popleft()
-            have_more = packet is not None and (len(self.ordinary_packets)>0 or len(self.packet_queue)>0)
+            have_more = packet is not None and (self.ordinary_packets or self.packet_queue)
         return packet, start_send_cb, end_send_cb, fail_cb, synchronous, have_more, will_have_more
 
     def send(self, *parts, **kwargs):
@@ -408,12 +429,6 @@ class ClientConnection(ClientConnectionClass):
         self.send(*parts, **kwargs)
 
 
-    #client tells us about network connection status:
-    def update_connection_data(self, data):
-        netlog("update_connection_data(%s)", data)
-        self.client_connection_data = data
-
-
     def send_hello(self, server_capabilities):
         capabilities = server_capabilities.copy()
         for bc in CC_BASES:
@@ -433,7 +448,6 @@ class ClientConnection(ClientConnectionClass):
                 "hello-sent"        : self.hello_sent,
                 "jitter"            : self.jitter,
                 "bandwidth-limit"   : {
-                    "setting"       : self.bandwidth_limit or 0,
                     "detection"     : self.bandwidth_detection,
                     "actual"        : self.soft_bandwidth_limit or 0,
                     }
@@ -482,8 +496,9 @@ class ClientConnection(ClientConnectionClass):
 
     ######################################################################
     # notifications:
-    """ Utility functions for mixins (makes notifications optional) """
-    def may_notify(self, nid, summary, body, actions=[], hints={}, expire_timeout=10*1000, icon_name=None, user_callback=None):
+    # Utility functions for mixins (makes notifications optional)
+    def may_notify(self, nid, summary, body, actions=(), hints=None, expire_timeout=10*1000,
+                   icon_name=None, user_callback=None):
         try:
             from xpra.platform.paths import get_icon_filename
             from xpra.notifications.common import parse_image_path
@@ -492,9 +507,12 @@ class ClientConnection(ClientConnectionClass):
         else:
             icon_filename = get_icon_filename(icon_name)
             icon = parse_image_path(icon_filename) or ""
-            self.notify("", nid, "Xpra", 0, "", summary, body, actions, hints, expire_timeout, icon, user_callback)
+            self.notify("", nid, "Xpra", 0, "",
+                        summary, body, actions, hints or {},
+                        expire_timeout, icon, user_callback)
 
-    def notify(self, dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon, user_callback=None):
+    def notify(self, dbus_id, nid, app_name, replaces_nid, app_icon,
+               summary, body, actions, hints, expire_timeout, icon, user_callback=None):
         args = (dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon)
         notifylog("notify%s types=%s", args, tuple(type(x) for x in args))
         if not self.send_notifications:
@@ -508,15 +526,16 @@ class ClientConnection(ClientConnectionClass):
         #this is one of the few places where we actually do care about character encoding:
         try:
             summary = summary.encode("utf8")
-        except:
+        except UnicodeEncodeError:
             summary = str(summary)
         try:
             body = body.encode("utf8")
-        except:
+        except UnicodeEncodeError:
             body = str(body)
         if self.hello_sent:
             #Warning: actions and hints are send last because they were added later (in version 2.3)
-            self.send_async("notify_show", dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, expire_timeout, icon, actions, hints)
+            self.send_async("notify_show", dbus_id, nid, app_name, replaces_nid, app_icon,
+                            summary, body, expire_timeout, icon, actions, hints)
         return True
 
     def notify_close(self, nid):
