@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,21 +9,21 @@ import os
 from math import sqrt
 from time import sleep
 
+from xpra.server.source.stub_source_mixin import StubSourceMixin
+from xpra.server.window.batch_config import DamageBatchConfig
+from xpra.server.server_core import ClientException
+from xpra.codecs.video_helper import getVideoHelper
+from xpra.codecs.codec_constants import video_spec
+from xpra.net.compression import use_lz4, use_lzo, use_brotli
+from xpra.os_util import monotonic_time, strtobytes
+from xpra.server.background_worker import add_work_item
+from xpra.util import csv, typedict, envint
 from xpra.log import Logger
+
 log = Logger("encoding")
 proxylog = Logger("proxy")
 statslog = Logger("stats")
 
-
-from xpra.server.source.stub_source_mixin import StubSourceMixin
-from xpra.server.window.batch_config import DamageBatchConfig
-from xpra.codecs.video_helper import getVideoHelper
-from xpra.codecs.codec_constants import video_spec
-from xpra.net.compression import compressed_wrapper, Compressed, use_lz4, use_lzo
-from xpra.os_util import monotonic_time
-from xpra.server.background_worker import add_work_item
-from xpra.util import csv, typedict, envint
-
 MIN_PIXEL_RECALCULATE = envint("XPRA_MIN_PIXEL_RECALCULATE", 2000)
 
 
@@ -44,7 +44,8 @@ class EncodingsMixin(StubSourceMixin):
         self.default_speed = 40         #encoding speed (only used by x264)
         self.default_min_speed = 10     #default minimum encoding speed
 
-        self.default_batch_config = DamageBatchConfig()     #contains default values, some of which may be supplied by the client
+        #contains default values, some of which may be supplied by the client:
+        self.default_batch_config = DamageBatchConfig()
         self.global_batch_config = self.default_batch_config.clone()      #global batch config
 
         self.vrefresh = -1
@@ -95,6 +96,12 @@ class EncodingsMixin(StubSourceMixin):
         self.video_helper.cleanup()
 
 
+    def all_window_sources(self):
+        #we can't assume that the window mixin is loaded:
+        window_sources = getattr(self, "window_sources", {})
+        return tuple(window_sources.values())
+
+
     def get_caps(self):
         caps = {}
         if self.wants_encodings and self.encoding:
@@ -106,17 +113,6 @@ class EncodingsMixin(StubSourceMixin):
         return caps
 
 
-    def compressed_wrapper(self, datatype, data, min_saving=128):
-        if self.zlib or self.lz4 or self.lzo:
-            cw = compressed_wrapper(datatype, data, zlib=self.zlib, lz4=self.lz4, lzo=self.lzo, can_inline=False)
-            if len(cw)+min_saving<=len(data):
-                #the compressed version is smaller, use it:
-                return cw
-            #skip compressed version: fall through
-        #we can't compress, so at least avoid warnings in the protocol layer:
-        return Compressed(datatype, data, can_inline=True)
-
-
     def recalculate_delays(self):
         """ calls update_averages() on ServerSource.statistics (GlobalStatistics)
             and WindowSource.statistics (WindowPerformanceStatistics) for each window id in calculate_window_ids,
@@ -133,21 +129,25 @@ class EncodingsMixin(StubSourceMixin):
         conn = p._conn
         if not conn:
             return
-        self.statistics.bytes_sent.append((now, conn.output_bytecount))
-        self.statistics.update_averages()
+        #we can't assume that 'self' is a full ClientConnection object:
+        stats = getattr(self, "statistics", None)
+        if stats:
+            stats.bytes_sent.append((now, conn.output_bytecount))
+            stats.update_averages()
         self.update_bandwidth_limits()
         wids = tuple(self.calculate_window_ids)  #make a copy so we don't clobber new wids
         focus = self.get_focus()
         sources = self.window_sources.items()
-        maximized_wids = [wid for wid, source in sources if source is not None and source.maximized]
-        fullscreen_wids = [wid for wid, source in sources if source is not None and source.fullscreen]
-        log("recalculate_delays() wids=%s, focus=%s, maximized=%s, fullscreen=%s", wids, focus, maximized_wids, fullscreen_wids)
+        maximized_wids = tuple(wid for wid, source in sources if source is not None and source.maximized)
+        fullscreen_wids = tuple(wid for wid, source in sources if source is not None and source.fullscreen)
+        log("recalculate_delays() wids=%s, focus=%s, maximized=%s, fullscreen=%s",
+            wids, focus, maximized_wids, fullscreen_wids)
         for wid in wids:
             #this is safe because we only add to this set from other threads:
             self.calculate_window_ids.remove(wid)
             try:
                 del self.calculate_window_pixels[wid]
-            except:
+            except KeyError:
                 pass
             ws = self.window_sources.get(wid)
             if ws is None:
@@ -158,7 +158,7 @@ class EncodingsMixin(StubSourceMixin):
                                          len(fullscreen_wids)>0 and wid not in fullscreen_wids,
                                          len(maximized_wids)>0 and wid not in maximized_wids)
                 ws.reconfigure()
-            except:
+            except Exception:
                 log.error("error on window %s", wid, exc_info=True)
             if self.is_closed():
                 return
@@ -174,17 +174,22 @@ class EncodingsMixin(StubSourceMixin):
             tsize += w*h
             tcount += 1
             time_w = 2.0+(now-ws.batch_config.last_updated)     #add 2 seconds to even things out
-            weight = w*h*time_w
+            weight = int(w*h*time_w)
             wdelay += ws.batch_config.delay*weight
             wdimsum += weight
         if wdimsum>0 and tcount>0:
             #weighted delay:
-            avg_size = tsize/tcount
-            wdelay = wdelay / wdimsum
-            #store the delay as a normalized value per megapixel:
-            delay = wdelay * 1000000 / avg_size
+            delay = wdelay // wdimsum
             self.global_batch_config.last_delays.append((now, delay))
             self.global_batch_config.delay = delay
+            #store the delay as a normalized value per megapixel
+            #so we can adapt it to different window sizes:
+            avg_size = tsize // tcount
+            ratio = sqrt(1000000.0 / avg_size)
+            normalized_delay = int(delay * ratio)
+            self.global_batch_config.delay_per_megapixel = normalized_delay
+            log("delay_per_megapixel=%i, delay=%i, for wdelay=%i, avg_size=%i, ratio=%.2f",
+                normalized_delay, delay, wdelay, avg_size, ratio)
 
     def may_recalculate(self, wid, pixel_count):
         if wid in self.calculate_window_ids:
@@ -203,7 +208,8 @@ class EncodingsMixin(StubSourceMixin):
         if delta>RECALCULATE_DELAY:
             add_work_item(self.recalculate_delays)
         else:
-            self.calculate_timer = self.timeout_add(int(1000*(RECALCULATE_DELAY-delta)), add_work_item, self.recalculate_delays)
+            delay = int(1000*(RECALCULATE_DELAY-delta))
+            self.calculate_timer = self.timeout_add(delay, add_work_item, self.recalculate_delays)
 
     def cancel_recalculate_timer(self):
         ct = self.calculate_timer
@@ -220,8 +226,8 @@ class EncodingsMixin(StubSourceMixin):
                 if value is not None:
                     try:
                         return int(value)
-                    except:
-                        log.error("invalid value for batch option %s: %s", varname, value)
+                    except (TypeError, ValueError):
+                        log.error("Error: invalid value '%s' for batch option %s", value, varname)
                 return None
             #from client caps first:
             cpname = "batch.%s" % prop
@@ -244,6 +250,9 @@ class EncodingsMixin(StubSourceMixin):
         self.zlib = c.boolget("zlib", True)
         self.lz4 = c.boolget("lz4", False) and use_lz4
         self.lzo = c.boolget("lzo", False) and use_lzo
+        self.brotli = c.boolget("brotli", False) and use_brotli
+        log("compressors: zlib=%s, lz4=%s, lzo=%s, brotli=%s",
+            self.zlib, self.lz4, self.lzo, self.brotli)
 
         self.vrefresh = c.intget("vrefresh", -1)
 
@@ -261,14 +270,16 @@ class EncodingsMixin(StubSourceMixin):
         #encodings:
         self.encodings = c.strlistget("encodings")
         self.core_encodings = c.strlistget("encodings.core", self.encodings)
-        if self.send_windows and not self.core_encodings:
-            raise Exception("client failed to specify any supported encodings")
-        if "png" in self.core_encodings:
-            self.window_icon_encodings.append("png")
+        log("encodings=%s, core_encodings=%s", self.encodings, self.core_encodings)
+        #we can't assume that the window mixin is loaded,
+        #or that the ui_client flag exists:
+        send_ui = getattr(self, "ui_client", True) and getattr(self, "send_windows", True)
+        if send_ui and not self.core_encodings:
+            raise ClientException("client failed to specify any supported encodings")
         self.window_icon_encodings = c.strlistget("encodings.window-icon", ["premult_argb32"])
         self.rgb_formats = c.strlistget("encodings.rgb_formats", ["RGB"])
         #skip all other encoding related settings if we don't send pixels:
-        if not self.send_windows:
+        if not send_ui:
             log("windows/pixels forwarding is disabled for this client")
         else:
             self.parse_encoding_caps(c)
@@ -291,8 +302,10 @@ class EncodingsMixin(StubSourceMixin):
                 self.encoding_options[ek] = c.boolget(k)
         #2: standardized encoding options:
         for k in c.keys():
-            if k.startswith(b"theme.") or  k.startswith(b"encoding.icons."):
-                self.icons_encoding_options[k.replace(b"encoding.icons.", b"").replace(b"theme.", b"")] = c[k]
+            #yaml gives us str..
+            k = strtobytes(k)
+            if k.startswith(b"theme.") or k.startswith(b"encoding.icons."):
+                self.icons_encoding_options[k.replace(b"encoding.icons.", b"").replace(b"theme.", b"")] = c.get(k)
             elif k.startswith(b"encoding."):
                 stripped_k = k[len(b"encoding."):]
                 if stripped_k in (b"transparency",
@@ -316,7 +329,7 @@ class EncodingsMixin(StubSourceMixin):
             #enabling video proxy:
             try:
                 self.parse_proxy_video()
-            except:
+            except Exception:
                 proxylog.error("failed to parse proxy video", exc_info=True)
 
         sc = self.encoding_options.get("scaling.control", self.scaling_control)
@@ -340,7 +353,8 @@ class EncodingsMixin(StubSourceMixin):
         self.auto_refresh_delay = c.intget("auto_refresh_delay", 0)
         #check for mmap:
         if getattr(self, "mmap_size", 0)==0:
-            others = [x for x in self.core_encodings if x in self.server_core_encodings and x!=self.encoding]
+            others = tuple(x for x in self.core_encodings
+                           if x in self.server_core_encodings and x!=self.encoding)
             if self.encoding=="auto":
                 s = "automatic picture encoding enabled"
             else:
@@ -360,8 +374,23 @@ class EncodingsMixin(StubSourceMixin):
             for colorspace, spec_props in colorspace_specs.items():
                 for spec_prop in spec_props:
                     #make a new spec based on spec_props:
-                    spec = video_spec(codec_class=Encoder, codec_type="proxy", encoding=encoding)
+                    spec_prop = typedict(spec_prop)
+                    input_colorspace = spec_prop.strget("input_colorspace")
+                    output_colorspaces = spec_prop.strlistget("output_colorspaces")
+                    if not input_colorspace or not output_colorspaces:
+                        log.warn("Warning: invalid proxy video encoding '%s':", encoding)
+                        log.warn(" missing colorspace attributes")
+                        continue
+                    spec = video_spec(codec_class=Encoder,
+                                      has_lossless_mode=spec_prop.boolget("has_lossless_mode", False),
+                                      input_colorspace=input_colorspace,
+                                      output_colorspaces=output_colorspaces,
+                                      codec_type="proxy", encoding=encoding,
+                                      )
                     for k,v in spec_prop.items():
+                        if k.startswith("_") or not hasattr(spec, k):
+                            log.warn("Warning: invalid proxy codec attribute '%s'", k)
+                            continue
                         setattr(spec, k, v)
                     proxylog("parse_proxy_video() adding: %s / %s / %s", encoding, colorspace, spec)
                     self.video_helper.add_encoder_spec(encoding, colorspace, spec)
@@ -375,7 +404,7 @@ class EncodingsMixin(StubSourceMixin):
         if window_ids is not None:
             wss = (self.window_sources.get(wid) for wid in window_ids)
         else:
-            wss = self.window_sources.values()
+            wss = self.all_window_sources()
         for ws in wss:
             if ws is not None:
                 ws.set_auto_refresh_delay(delay)
@@ -385,8 +414,6 @@ class EncodingsMixin(StubSourceMixin):
             or for all windows if 'window_ids' is None.
         """
         log("set_encoding(%s, %s, %s)", encoding, window_ids, strict)
-        if not self.ui_client:
-            return
         if encoding and encoding!="auto":
             #old clients (v0.9.x and earlier) only supported 'rgb24' as 'rgb' mode:
             if encoding=="rgb24":
@@ -402,11 +429,14 @@ class EncodingsMixin(StubSourceMixin):
         if window_ids is not None:
             wss = [self.window_sources.get(wid) for wid in window_ids]
         else:
-            wss = self.window_sources.values()
+            wss = self.all_window_sources()
         #if we're updating all the windows, reset global stats too:
-        if set(wss).issuperset(self.window_sources.values()):
+        if set(wss).issuperset(self.all_window_sources()):
             log("resetting global stats")
-            self.statistics.reset()
+            #we can't assume that 'self' is a full ClientConnection object:
+            stats = getattr(self, "statistics", None)
+            if stats:
+                stats.reset()
             self.global_batch_config = self.default_batch_config.clone()
         for ws in wss:
             if ws is not None:
@@ -425,7 +455,7 @@ class EncodingsMixin(StubSourceMixin):
         ieo = dict(self.icons_encoding_options)
         try:
             del ieo["default.icons"]
-        except:
+        except KeyError:
             pass
         #encoding:
         info.update({
@@ -446,19 +476,19 @@ class EncodingsMixin(StubSourceMixin):
 
 
     def set_min_quality(self, min_quality):
-        for ws in tuple(self.window_sources.values()):
+        for ws in tuple(self.all_window_sources()):
             ws.set_min_quality(min_quality)
 
     def set_quality(self, quality):
-        for ws in tuple(self.window_sources.values()):
+        for ws in tuple(self.all_window_sources()):
             ws.set_quality(quality)
 
     def set_min_speed(self, min_speed):
-        for ws in tuple(self.window_sources.values()):
+        for ws in tuple(self.all_window_sources()):
             ws.set_min_speed(min_speed)
 
     def set_speed(self, speed):
-        for ws in tuple(self.window_sources.values()):
+        for ws in tuple(self.all_window_sources()):
             ws.set_speed(speed)
 
 
@@ -483,7 +513,11 @@ class EncodingsMixin(StubSourceMixin):
         #but use sqrt to smooth things and prevent excesses
         #(ie: a 4MPixel window, will start at 2 times the global delay)
         #(ie: a 0.5MPixel window will start at 0.7 times the global delay)
+        dpm = self.global_batch_config.delay_per_megapixel
         w, h = window.get_dimensions()
-        ratio = float(w*h) / 1000000
-        batch_config.delay = self.global_batch_config.delay * sqrt(ratio)
+        if dpm>=0:
+            ratio = sqrt(1000000.0 / (w*h))
+            batch_config.delay = max(batch_config.min_delay, min(batch_config.max_delay, int(dpm * sqrt(ratio))))
+        log("make_batch_config(%i, %s) global delay per megapixel=%i, new window delay for %ix%i=%s",
+                 wid, window, dpm, w, h, batch_config.delay)
         return batch_config
