@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,15 +10,20 @@ from threading import Lock
 
 from xpra.sound.sound_pipeline import SoundPipeline
 from xpra.gtk_common.gobject_util import one_arg_signal, gobject
-from xpra.sound.gstreamer_util import plugin_str, get_decoder_elements, get_queue_time, normv, get_decoders, get_default_sink, get_sink_plugins, \
-                                        MP3, CODEC_ORDER, gst, QUEUE_LEAK, GST_QUEUE_NO_LEAK, MS_TO_NS, DEFAULT_SINK_PLUGIN_OPTIONS
+from xpra.sound.gstreamer_util import (
+    plugin_str, get_decoder_elements,
+    get_queue_time, normv, get_decoders,
+    get_default_sink, get_sink_plugins,
+    MP3, CODEC_ORDER, gst, QUEUE_LEAK,
+    GST_QUEUE_NO_LEAK, MS_TO_NS, DEFAULT_SINK_PLUGIN_OPTIONS,
+    )
 from xpra.gtk_common.gobject_compat import import_glib
 from xpra.net.compression import decompress_by_name
-
 from xpra.scripts.config import InitExit
 from xpra.util import csv, envint, envbool
 from xpra.os_util import thread, monotonic_time
 from xpra.log import Logger
+
 log = Logger("sound")
 gstlog = Logger("gstreamer")
 
@@ -61,7 +66,7 @@ class SoundSink(SoundPipeline):
         "eos"       : one_arg_signal,
         })
 
-    def __init__(self, sink_type=None, sink_options={}, codecs=get_decoders(), codec_options={}, volume=1.0):
+    def __init__(self, sink_type=None, sink_options=None, codecs=(), codec_options=None, volume=1.0):
         if not sink_type:
             sink_type = get_default_sink()
         if sink_type not in get_sink_plugins():
@@ -69,14 +74,16 @@ class SoundSink(SoundPipeline):
         matching = [x for x in CODEC_ORDER if (x in codecs and x in get_decoders())]
         log("SoundSink(..) found matching codecs %s", matching)
         if not matching:
-            raise InitExit(1, "no matching codecs between arguments '%s' and supported list '%s'" % (csv(codecs), csv(get_decoders().keys())))
+            raise InitExit(1, "no matching codecs between arguments '%s' and supported list '%s'" % (
+                csv(codecs), csv(get_decoders().keys())))
         codec = matching[0]
         decoder, parser, stream_compressor = get_decoder_elements(codec)
         SoundPipeline.__init__(self, codec)
         self.container_format = (parser or "").replace("demux", "").replace("depay", "")
         self.sink_type = sink_type
         self.stream_compressor = stream_compressor
-        log("container format=%s, stream_compressor=%s, sink type=%s", self.container_format, self.stream_compressor, self.sink_type)
+        log("container format=%s, stream_compressor=%s, sink type=%s",
+            self.container_format, self.stream_compressor, self.sink_type)
         self.levels = deque(maxlen=100)
         self.volume = None
         self.src    = None
@@ -130,7 +137,8 @@ class SoundSink(SoundPipeline):
             v = get_options_cb()
             log("%s()=%s", get_options_cb, v)
             sink_attributes.update(v)
-        sink_attributes.update(sink_options)
+        if sink_options:
+            sink_attributes.update(sink_options)
         sink_str = plugin_str(sink_type, sink_attributes)
         pipeline_els.append(sink_str)
         if not self.setup_pipeline_and_bus(pipeline_els):
@@ -266,7 +274,8 @@ class SoundSink(SoundPipeline):
             mst = self.queue.get_property("max-size-time")//MS_TO_NS
             mrange = max(lrange+100, UNDERRUN_MIN_LEVEL)
             mtt = min(mst-50, mrange)
-            gstlog("set_min_level mtt=%3i, max-size-time=%3i, lrange=%s, mrange=%s (UNDERRUN_MIN_LEVEL=%s)", mtt, mst, lrange, mrange, UNDERRUN_MIN_LEVEL)
+            gstlog("set_min_level mtt=%3i, max-size-time=%3i, lrange=%s, mrange=%s (UNDERRUN_MIN_LEVEL=%s)",
+                   mtt, mst, lrange, mrange, UNDERRUN_MIN_LEVEL)
         else:
             mtt = 0
         cmtt = self.queue.get_property("min-threshold-time")//MS_TO_NS
@@ -309,7 +318,8 @@ class SoundSink(SoundPipeline):
             mst += UNDERRUN_MIN_LEVEL
         #cap it at 1 second:
         mst = min(mst, 1000)
-        log("set_max_level overrun count=%-2i, margin=%3i, pct=%2i, cmst=%3i, target=%3i, mst=%3i", olm, MARGIN, pct, cmst, target_mst, mst)
+        log("set_max_level overrun count=%-2i, margin=%3i, pct=%2i, cmst=%3i, target=%3i, mst=%3i",
+            olm, MARGIN, pct, cmst, target_mst, mst)
         if abs(cmst-mst)<=max(50, lrange//2):
             #not enough difference
             return
@@ -385,7 +395,7 @@ class SoundSink(SoundPipeline):
                 clt = self.queue.get_property("current-level-time")//MS_TO_NS
                 qmin = self.queue.get_property("min-threshold-time")//MS_TO_NS
                 gstlog("add_data: refill=%s, level=%i, min=%i", self.refill, clt, qmin)
-                if qmin>0 and clt>qmin:
+                if 0<qmin<clt:
                     self.refill = False
         self.emit_info()
 
@@ -498,16 +508,14 @@ def main():
                 sys.exit()
             signal.signal(signal.SIGINT, force_quit)
             signal.signal(signal.SIGTERM, force_quit)
-        from xpra.gtk_common.gobject_compat import is_gtk3
-        if not is_gtk3():
-            signal.signal(signal.SIGINT, deadly_signal)
+        signal.signal(signal.SIGINT, deadly_signal)
         signal.signal(signal.SIGTERM, deadly_signal)
 
         def check_for_end(*_args):
             qtime = ss.queue.get_property("current-level-time")//MS_TO_NS
             if qtime<=0:
                 log.info("underrun (end of stream)")
-                thread.start_new_thread(ss.stop, ())
+                thread.start_new_thread(ss.stop, ())        #@UndefinedVariable
                 glib.timeout_add(500, glib_mainloop.quit)
                 return False
             return True
