@@ -38,7 +38,7 @@ def full_version_str():
     try:
         from xpra.src_info import REVISION, LOCAL_MODIFICATIONS
         s += "-r%s%s" % (REVISION, "M" if LOCAL_MODIFICATIONS>0 else "")
-    except ImportError:
+    except (ImportError, TypeError):
         pass
     return s
 
@@ -152,7 +152,11 @@ def do_get_platform_info():
     ld = get_linux_distribution()
     if ld:
         info["linux_distribution"] = ld
-    release = platform_release(pp.release())
+    try:
+        release = platform_release(pp.release())
+    except OSError:
+        log("do_get_platform_info()", exc_info=True)
+        release = "unknown"
     info.update({
             ""          : sys.platform,
             "name"      : platform_name(sys.platform, info.get("linux_distribution") or release),
