@@ -1,41 +1,56 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import gtk
-import gobject
 import os
 
+from xpra.util import envbool
+from xpra.os_util import PYTHON2
 from xpra.gtk_common.error import xsync, xswallow
-import xpra.x11.gtk_x11.selection
-from xpra.x11.gtk_x11.world_window import WorldWindow
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
+from xpra.x11.window_info import window_name, window_info
 from xpra.gtk_common.gobject_util import no_arg_signal, one_arg_signal
-
-from xpra.x11.gtk_x11.window import WindowModel, Unmanageable
+from xpra.gtk_common.gtk_util import (
+    get_default_root_window, display_get_default, get_xwindow,
+    GDKWINDOW_FOREIGN, CLASS_INPUT_ONLY, GDKWindow,
+    PARAM_READABLE,
+    )
+from xpra.x11.common import Unmanageable, MAX_WINDOW_SIZE
+from xpra.x11.gtk_x11.selection import ManagerSelection
+from xpra.x11.gtk_x11.world_window import WorldWindow
+from xpra.x11.models.window import WindowModel, configure_bits
 from xpra.x11.gtk_x11.gdk_bindings import (
-               add_event_receiver,                          #@UnresolvedImport
-               get_children,                                #@UnresolvedImport
-               )
+    add_event_receiver,                              #@UnresolvedImport
+    add_fallback_receiver, remove_fallback_receiver, #@UnresolvedImport
+    get_children,                                    #@UnresolvedImport
+    )
 from xpra.x11.bindings.window_bindings import constants, X11WindowBindings #@UnresolvedImport
-X11Window = X11WindowBindings()
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
+from xpra.gtk_common.gobject_compat import import_gobject, is_gtk3
+from xpra.log import Logger
+
+log = Logger("x11", "window")
+
+gobject = import_gobject()
+
+X11Window = X11WindowBindings()
 X11Keyboard = X11KeyboardBindings()
 
-from xpra.log import Logger
-log = Logger("x11", "window")
 focuslog = Logger("x11", "window", "focus")
 screenlog = Logger("x11", "window", "screen")
+framelog = Logger("x11", "window", "frame")
 
+CWX             = constants["CWX"]
+CWY             = constants["CWY"]
+CWWidth         = constants["CWWidth"]
+CWHeight        = constants["CWHeight"]
 
 NotifyPointerRoot   = constants["NotifyPointerRoot"]
 NotifyDetailNone    = constants["NotifyDetailNone"]
 
-FORCE_REPLACE_WM = os.environ.get("XPRA_FORCE_REPLACE_WM", "0")=="1"
-LOG_MANAGE_FAILURES = os.environ.get("XPRA_LOG_MANAGE_FAILURES", "0")=="1"
-
+LOG_MANAGE_FAILURES = envbool("XPRA_LOG_MANAGE_FAILURES", False)
 
 NO_NET_SUPPORTED = os.environ.get("XPRA_NO_NET_SUPPORTED", "").split(",")
 
@@ -87,12 +102,10 @@ DEFAULT_NET_SUPPORTED = [
         "_NET_CLOSE_WINDOW",
 
         # These aren't supported in any particularly meaningful way, but hey.
-        "_NET_FRAME_EXTENTS",
-
         "_NET_WM_WINDOW_TYPE",
         "_NET_WM_WINDOW_TYPE_NORMAL",
-        # "_NET_WM_WINDOW_TYPE_DESKTOP",
-        # "_NET_WM_WINDOW_TYPE_DOCK",
+        "_NET_WM_WINDOW_TYPE_DESKTOP",
+        "_NET_WM_WINDOW_TYPE_DOCK",
         "_NET_WM_WINDOW_TYPE_TOOLBAR",
         "_NET_WM_WINDOW_TYPE_MENU",
         "_NET_WM_WINDOW_TYPE_UTILITY",
@@ -101,7 +114,7 @@ DEFAULT_NET_SUPPORTED = [
         "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU",
         "_NET_WM_WINDOW_TYPE_POPUP_MENU",
         "_NET_WM_WINDOW_TYPE_TOOLTIP",
-        # "_NET_WM_WINDOW_TYPE_NOTIFICATION",
+        "_NET_WM_WINDOW_TYPE_NOTIFICATION",
         "_NET_WM_WINDOW_TYPE_COMBO",
         # "_NET_WM_WINDOW_TYPE_DND",
 
@@ -119,62 +132,32 @@ DEFAULT_NET_SUPPORTED = [
         "_NET_WM_STATE_FULLSCREEN",
         "_NET_WM_STATE_ABOVE",
         "_NET_WM_STATE_BELOW",
+        "_NET_WM_STATE_FOCUSED",
+
+        "_NET_WM_DESKTOP",
 
         "_NET_WM_MOVERESIZE",
         "_NET_MOVERESIZE_WINDOW",
 
         "_MOTIF_WM_HINTS",
         "_MOTIF_WM_INFO",
+
+        "_NET_REQUEST_FRAME_EXTENTS",
         # Not at all yet:
-        #"_NET_REQUEST_FRAME_EXTENTS",
         #"_NET_RESTACK_WINDOW",
-        #"_NET_WM_DESKTOP",
         ]
+FRAME_EXTENTS = envbool("XPRA_FRAME_EXTENTS", True)
+if FRAME_EXTENTS:
+    DEFAULT_NET_SUPPORTED.append("_NET_FRAME_EXTENTS")
+
 NET_SUPPORTED = [x for x in DEFAULT_NET_SUPPORTED if x not in NO_NET_SUPPORTED]
 
+def u(s):
+    if PYTHON2:
+        return s.decode("utf8")
+    return s
 
-def wm_check(display, wm_name, upgrading=False):
-    #there should only be one screen... but let's check all of them
-    for i in range(display.get_n_screens()):
-        screen = display.get_screen(i)
-        root = screen.get_root_window()
-        wm_prop = "WM_S%s" % i
-        cwm_prop = "_NEW_WM_CM_S%s" % i
-        wm_so = X11Window.XGetSelectionOwner(wm_prop)
-        cwm_so = X11Window.XGetSelectionOwner(cwm_prop)
-        log("ewmh selection owner for %s: %s", wm_prop, wm_so)
-        log("compositing window manager %s: %s", cwm_prop, cwm_so)
-
-        try:
-            ewmh_wm = prop_get(root, "_NET_SUPPORTING_WM_CHECK", "window", ignore_errors=True, raise_xerrors=False)
-        except:
-            #errors here generally indicate that the window is gone
-            #which is fine: it means the previous window manager is no longer active
-            continue
-        def xid(w):
-            if w:
-                return "%#x" % w.xid
-            return None
-        log("_NET_SUPPORTING_WM_CHECK for screen %s: %s (root=%s)", i, xid(ewmh_wm), xid(root))
-        if ewmh_wm:
-            try:
-                name = prop_get(ewmh_wm, "_NET_WM_NAME", "utf8", ignore_errors=False, raise_xerrors=False)
-            except:
-                name = None
-            if upgrading and name and name==wm_name:
-                log.info("found previous Xpra instance")
-            else:
-                log.warn("Warning: found an existing window manager on screen %s using window %#x: %s", i, ewmh_wm.xid, name or "unknown")
-            if (wm_so is None or wm_so==0) and (cwm_so is None or cwm_so==0):
-                if FORCE_REPLACE_WM:
-                    log.warn("XPRA_FORCE_REPLACE_WM is set, replacing it forcibly")
-                else:
-                    log.error("it does not own the selection '%s' or '%s' so we cannot take over and make it exit", wm_prop, cwm_prop)
-                    log.error("please stop %s so you can run xpra on this display", name or "the existing window manager")
-                    log.warn("if you are certain that the window manager is already gone,")
-                    log.warn(" you may set XPRA_FORCE_REPLACE_WM=1 to force xpra to continue, at your own risk")
-                    return False
-    return True
+DEFAULT_SIZE_CONSTRAINTS = (0, 0, MAX_WINDOW_SIZE, MAX_WINDOW_SIZE)
 
 
 class Wm(gobject.GObject):
@@ -182,17 +165,15 @@ class Wm(gobject.GObject):
     __gproperties__ = {
         "windows": (gobject.TYPE_PYOBJECT,
                     "Set of managed windows (as WindowModels)", "",
-                    gobject.PARAM_READABLE),
+                    PARAM_READABLE),
         "toplevel": (gobject.TYPE_PYOBJECT,
                      "Toplevel container widget for the display", "",
-                     gobject.PARAM_READABLE),
+                     PARAM_READABLE),
         }
     __gsignals__ = {
         # Public use:
         # A new window has shown up:
         "new-window": one_arg_signal,
-        # X11 bell event:
-        "bell": one_arg_signal,
         "show-desktop": one_arg_signal,
         # You can emit this to cause the WM to quit, or the WM may
         # spontaneously raise it if another WM takes over the display.  By
@@ -212,7 +193,7 @@ class Wm(gobject.GObject):
         gobject.GObject.__init__(self)
 
         if display is None:
-            display = gtk.gdk.display_manager_get().get_default_display()
+            display = display_get_default()
         self._display = display
         self._root = self._display.get_default_screen().get_root_window()
         self._wm_name = wm_name
@@ -224,8 +205,8 @@ class Wm(gobject.GObject):
         self._windows_in_order = []
 
         # Become the Official Window Manager of this year's display:
-        self._wm_selection = xpra.x11.gtk_x11.selection.ManagerSelection(self._display, "WM_S0")
-        self._cm_wm_selection = xpra.x11.gtk_x11.selection.ManagerSelection(self._display, "_NET_WM_CM_S0")
+        self._wm_selection = ManagerSelection("WM_S0")
+        self._cm_wm_selection = ManagerSelection("_NET_WM_CM_S0")
         self._wm_selection.connect("selection-lost", self._lost_wm_selection)
         self._cm_wm_selection.connect("selection-lost", self._lost_wm_selection)
         # May throw AlreadyOwned:
@@ -239,37 +220,48 @@ class Wm(gobject.GObject):
         # Set up the necessary EWMH properties on the root window.
         self._setup_ewmh_window()
         # Start with just one desktop:
-        self.set_desktop_list([u"Main"])
+        self.set_desktop_list((u"Main", ))
         self.set_current_desktop(0)
         # Start with the full display as workarea:
-        root_w, root_h = gtk.gdk.get_default_root_window().get_size()
+        root_w, root_h = get_default_root_window().get_geometry()[2:4]
         self.root_set("_NET_SUPPORTED", ["atom"], NET_SUPPORTED)
         self.set_workarea(0, 0, root_w, root_h)
         self.set_desktop_geometry(root_w, root_h)
         self.root_set("_NET_DESKTOP_VIEWPORT", ["u32"], [0, 0])
 
+        self.size_constraints = DEFAULT_SIZE_CONSTRAINTS
+
         # Load up our full-screen widget
-        self._world_window = WorldWindow(self._display.get_default_screen())
-        self.notify("toplevel")
-        self._world_window.show_all()
+        self._world_window = None
+        if not is_gtk3():
+            self._world_window = WorldWindow(self._display.get_default_screen())
+            self.notify("toplevel")
+            self._world_window.show_all()
 
         # Okay, ready to select for SubstructureRedirect and then load in all
         # the existing clients.
         add_event_receiver(self._root, self)
-        X11Window.substructureRedirect(self._root.xid)
+        add_fallback_receiver("xpra-client-message-event", self)
+        #when reparenting, the events may get sent
+        #to a window that is already destroyed
+        #and we don't want to miss those events, so:
+        add_fallback_receiver("child-map-request-event", self)
+        rxid = get_xwindow(self._root)
+        X11Window.substructureRedirect(rxid)
 
         for w in get_children(self._root):
             # Checking for FOREIGN here filters out anything that we've
             # created ourselves (like, say, the world window), and checking
             # for mapped filters out any withdrawn windows.
-            if (w.get_window_type() == gtk.gdk.WINDOW_FOREIGN
-                and not X11Window.is_override_redirect(w.xid)
-                and X11Window.is_mapped(w.xid)):
-                log("Wm managing pre-existing child")
+            xid = get_xwindow(w)
+            if (w.get_window_type() == GDKWINDOW_FOREIGN
+                and not X11Window.is_override_redirect(xid)
+                and X11Window.is_mapped(xid)):
+                log("Wm managing pre-existing child window %#x", xid)
                 self._manage_client(w)
 
         # Also watch for focus change events on the root window
-        X11Window.selectFocusChange(self._root.xid)
+        X11Window.selectFocusChange(rxid)
         X11Keyboard.selectBellNotification(True)
 
         # FIXME:
@@ -280,12 +272,15 @@ class Wm(gobject.GObject):
     def root_set(self, *args):
         prop_set(self._root, *args)
 
+    def root_get(self, *args):
+        return prop_get(self._root, *args)
+
     def set_dpi(self, xdpi, ydpi):
         #this is used by some newer versions of the dummy driver (xf86-driver-dummy)
         #(and will not be honoured by anything else..)
         self.root_set("dummy-constant-xdpi", "u32", xdpi)
         self.root_set("dummy-constant-ydpi", "u32", ydpi)
-        screenlog("set_dpi(%s, %s)", xdpi, ydpi)
+        screenlog("set_dpi(%i, %i)", xdpi, ydpi)
 
     def set_workarea(self, x, y, width, height):
         v = [x, y, width, height]
@@ -296,64 +291,72 @@ class Wm(gobject.GObject):
         v = [width, height]
         screenlog("_NET_DESKTOP_GEOMETRY=%s", v)
         self.root_set("_NET_DESKTOP_GEOMETRY", ["u32"], v)
+        #update all the windows:
+        for model in self._windows.values():
+            model.update_desktop_geometry(width, height)
+
+    def set_size_constraints(self, minw=0, minh=0, maxw=MAX_WINDOW_SIZE, maxh=MAX_WINDOW_SIZE):
+        log("set_size_constraints%s", (minw, minh, maxw, maxh))
+        self.size_constraints = minw, minh, maxw, maxh
+        #update all the windows:
+        for model in self._windows.values():
+            model.update_size_constraints(minw, minh, maxw, maxh)
 
 
-    def enableCursors(self, on):
-        log("enableCursors(%s)" % on)
-        X11Keyboard.selectCursorChange(on)
+    def set_default_frame_extents(self, v):
+        framelog("set_default_frame_extents(%s)", v)
+        if not v or len(v)!=4:
+            v = (0, 0, 0, 0)
+        self.root_set("DEFAULT_NET_FRAME_EXTENTS", ["u32"], v)
+        #update the models that are using the global default value:
+        for win in self._windows.values():
+            if win.is_OR() or win.is_tray():
+                continue
+            cur = win.get_property("frame")
+            if cur is None:
+                win._handle_frame_changed()
 
-    def do_xpra_xkb_event(self, event):
-        log("wm.do_xpra_xkb_event(%r)" % event)
-        if event.type!="bell":
-            log.error("wm.do_xpra_xkb_event(%r) unknown event type: %s" % (event, event.type))
-            return
-        self.do_bell_event(event)
-
-    def do_bell_event(self, event):
-        log("wm.do_bell_event(%s)", event)
-        self.emit("bell", event)
 
     def do_get_property(self, pspec):
         if pspec.name == "windows":
-            return frozenset(self._windows.itervalues())
-        elif pspec.name == "toplevel":
+            return frozenset(self._windows.values())
+        if pspec.name == "toplevel":
             return self._world_window
-        else:
-            assert False
+        assert False
 
     # This is in some sense the key entry point to the entire WM program.  We
     # have detected a new client window, and start managing it:
     def _manage_client(self, gdkwindow):
+        if not gdkwindow:
+            return
         if gdkwindow in self._windows:
             #already managed
             return
         try:
             with xsync:
-                self.do_manage_client(gdkwindow)
+                log("_manage_client(%s)", gdkwindow)
+                desktop_geometry = self.root_get("_NET_DESKTOP_GEOMETRY", ["u32"], True, False)
+                win = WindowModel(self._root, gdkwindow, desktop_geometry, self.size_constraints)
         except Exception as e:
-            if LOG_MANAGE_FAILURES or type(e) not in (Unmanageable, ):
+            if LOG_MANAGE_FAILURES or not isinstance(e, Unmanageable):
                 l = log.warn
             else:
                 l = log
-            l("failed to manage client %s: %s", gdkwindow, e, exc_info=True)
+            l("Warning: failed to manage client window %#x:", get_xwindow(gdkwindow))
+            l(" %s", e)
+            l("", exc_info=True)
+            with xswallow:
+                l(" window name: %s", window_name(gdkwindow))
+                l(" window info: %s", window_info(gdkwindow))
+        else:
+            win.managed_connect("unmanaged", self._handle_client_unmanaged)
+            self._windows[gdkwindow] = win
+            self._windows_in_order.append(gdkwindow)
+            self.notify("windows")
+            self._update_window_list()
+            self.emit("new-window", win)
 
-    def do_manage_client(self, gdkwindow):
-        try:
-            win = WindowModel(self._root, gdkwindow)
-        except Unmanageable:
-            log("Window disappeared on us, never mind")
-            return
-        win.connect("unmanaged", self._handle_client_unmanaged)
-        def bell_event(window_model, event):
-            self.do_bell_event(event)
-        win.connect("bell", bell_event)
-        self._windows[gdkwindow] = win
-        self._windows_in_order.append(gdkwindow)
-        self.notify("windows")
-        self._update_window_list()
-        self.emit("new-window", win)
-
-    def _handle_client_unmanaged(self, window, wm_exiting):
+    def _handle_client_unmanaged(self, window, _wm_exiting):
         gdkwindow = window.get_property("client-window")
         assert gdkwindow in self._windows
         del self._windows[gdkwindow]
@@ -361,7 +364,7 @@ class Wm(gobject.GObject):
         self._update_window_list()
         self.notify("windows")
 
-    def _update_window_list(self, *args):
+    def _update_window_list(self, *_args):
         # Ignore errors because not all the windows may still exist; if so,
         # then it's okay to leave the lists out of date for a moment, because
         # in a moment we'll get a signal telling us about the window that
@@ -377,16 +380,29 @@ class Wm(gobject.GObject):
         # Need to listen for:
         #   _NET_ACTIVE_WINDOW
         #   _NET_CURRENT_DESKTOP
-        #   _NET_REQUEST_FRAME_EXTENTS
         #   _NET_WM_PING responses
         # and maybe:
         #   _NET_RESTACK_WINDOW
-        #   _NET_WM_DESKTOP
         #   _NET_WM_STATE
         log("do_xpra_client_message_event(%s)", event)
         if event.message_type=="_NET_SHOWING_DESKTOP":
             show = bool(event.data[0])
             self.emit("show-desktop", show)
+        elif event.message_type=="_NET_REQUEST_FRAME_EXTENTS" and FRAME_EXTENTS:
+            #if we're here, that means the window model does not exist
+            #(or it would have processed the event)
+            #so this must be a an unmapped window
+            frame = (0, 0, 0, 0)
+            with xswallow:
+                if not X11Window.is_override_redirect(get_xwindow(event.window)):
+                    #use the global default:
+                    frame = prop_get(self._root, "DEFAULT_NET_FRAME_EXTENTS", ["u32"], ignore_errors=True)
+                if not frame:
+                    #fallback:
+                    frame = (0, 0, 0, 0)
+                framelog("_NET_REQUEST_FRAME_EXTENTS: setting _NET_FRAME_EXTENTS=%s on %#x",
+                         frame, get_xwindow(event.window))
+                prop_set(event.window, "_NET_FRAME_EXTENTS", ["u32"], frame)
 
     def _lost_wm_selection(self, selection):
         log.info("Lost WM selection %s, exiting", selection)
@@ -396,7 +412,9 @@ class Wm(gobject.GObject):
         self.cleanup()
 
     def cleanup(self):
-        for win in list(self._windows.itervalues()):
+        remove_fallback_receiver("xpra-client-message-event", self)
+        remove_fallback_receiver("child-map-request-event", self)
+        for win in tuple(self._windows.values()):
             win.unmanage(True)
 
     def do_child_map_request_event(self, event):
@@ -411,13 +429,32 @@ class Wm(gobject.GObject):
         # anyway, no harm in letting them move existing ones around), and it
         # means that when the window actually gets mapped, we have more
         # accurate info on what the app is actually requesting.
-        log("do_child_configure_request_event(%s)", event)
-        if event.window in self._windows:
+        model = self._windows.get(event.window)
+        if model:
+            #the window has been reparented already,
+            #but we're getting the configure request event on the root window
+            #forward it to the model
+            log("do_child_configure_request_event(%s) value_mask=%s, forwarding to %s",
+                event, configure_bits(event.value_mask), model)
+            model.do_child_configure_request_event(event)
             return
-        log("Reconfigure on withdrawn window")
+        log("do_child_configure_request_event(%s) value_mask=%s, reconfigure on withdrawn window",
+            event, configure_bits(event.value_mask))
         with xswallow:
-            X11Window.configureAndNotify(event.window.xid, event.x, event.y,
-                     event.width, event.height, event.value_mask)
+            xid = get_xwindow(event.window)
+            x, y, w, h = X11Window.getGeometry(xid)[:4]
+            if event.value_mask & CWX:
+                x = event.x
+            if event.value_mask & CWY:
+                y = event.y
+            if event.value_mask & CWWidth:
+                w = event.width
+            if event.value_mask & CWHeight:
+                h = event.height
+            if event.value_mask & (CWX | CWY | CWWidth | CWHeight):
+                log("updated window geometry for window %#x from %s to %s",
+                    xid, X11Window.getGeometry(xid)[:4], (x, y, w, h))
+            X11Window.configureAndNotify(xid, x, y, w, h, event.value_mask)
 
     def do_xpra_focus_in_event(self, event):
         # The purpose of this function is to detect when the focus mode has
@@ -425,7 +462,7 @@ class Wm(gobject.GObject):
         # something real.  This is easy to detect -- a FocusIn event with
         # detail PointerRoot or None is generated on the root window.
         focuslog("wm.do_xpra_focus_in_event(%s)", event)
-        if event.detail in (NotifyPointerRoot, NotifyDetailNone):
+        if event.detail in (NotifyPointerRoot, NotifyDetailNone) and self._world_window:
             self._world_window.reset_x_focus()
 
     def do_xpra_focus_out_event(self, event):
@@ -434,7 +471,7 @@ class Wm(gobject.GObject):
     def set_desktop_list(self, desktops):
         log("set_desktop_list(%s)", desktops)
         self.root_set("_NET_NUMBER_OF_DESKTOPS", "u32", len(desktops))
-        self.root_set("_NET_DESKTOP_NAMES", ["utf8"], [d.decode("utf8") for d in desktops])
+        self.root_set("_NET_DESKTOP_NAMES", ["utf8"], [u(d) for d in desktops])
 
     def set_current_desktop(self, index):
         self.root_set("_NET_CURRENT_DESKTOP", "u32", index)
@@ -452,17 +489,11 @@ class Wm(gobject.GObject):
         # clobber any XSelectInput calls that *we* might have wanted to make
         # on this window.)  Also, GDK might silently swallow all events that
         # are detected on it, anyway.
-        self._ewmh_window = gtk.gdk.Window(self._root,
-                                           width=1,
-                                           height=1,
-                                           window_type=gtk.gdk.WINDOW_TOPLEVEL,
-                                           event_mask=0, # event mask
-                                           wclass=gtk.gdk.INPUT_ONLY,
-                                           title=self._wm_name)
+        self._ewmh_window = GDKWindow(self._root, wclass=CLASS_INPUT_ONLY, title=self._wm_name)
         prop_set(self._ewmh_window, "_NET_SUPPORTING_WM_CHECK",
                  "window", self._ewmh_window)
         self.root_set("_NET_SUPPORTING_WM_CHECK", "window", self._ewmh_window)
-        self.root_set("_NET_WM_NAME", "utf8", self._wm_name.decode("utf8"))
+        self.root_set("_NET_WM_NAME", "utf8", u(self._wm_name))
 
     def get_net_wm_name(self):
         try:
