@@ -411,10 +411,12 @@ class KeyboardConfig(KeyboardConfigBase):
             log("set_default_keymap: modifier_map=%s", self.modifier_map)
 
 
-    def do_get_keycode(self, client_keycode, keyname, pressed, modifiers):
+    def do_get_keycode(self, client_keycode, keyname, pressed, modifiers, group):
         if not self.enabled:
             log("ignoring keycode since keyboard is turned off")
             return -1
+        if keyname=="0xffffff":
+            return -1
         keycode = None
         if self.xkbmap_query:
             keycode = self.keycode_translation.get((client_keycode, keyname)) or client_keycode
@@ -422,7 +424,7 @@ class KeyboardConfig(KeyboardConfigBase):
         else:
             #non-native: try harder to find matching keysym
             #first, try to honour shift state:
-            shift = "shift" in modifiers
+            shift = ("shift" in modifiers) ^ ("lock" in modifiers)
             mode = 0
             for mod in modifiers:
                 names = self.keynames_for_mod.get(mod, [])
@@ -431,10 +433,23 @@ class KeyboardConfig(KeyboardConfigBase):
                         mode = 1
                         break
             level = int(shift) + int(mode)*2
-            keycode = self.keycode_translation.get((keyname, level))
+            levels = []
+            if group:
+                levels.append(level+4)
+            levels.append(level)
+            for i in range(4):
+                level = int(shift) + i*2
+                if level not in levels:
+                    levels.append(level)
+            for level in levels:
+                keycode = self.keycode_translation.get((keyname, level))
+                if keycode:
+                    log("get_keycode(%s, %s, %s, %s)=%i (level=%i, shift=%s, mode=%i)",
+                        client_keycode, keyname, modifiers, group, keycode, level, shift, mode)
+                    break
             if keycode is None:
-                keycode = self.keycode_translation.get(keyname, client_keycode)
-            log("get_keycode(%s, %s, %s)=%i (level=%i)", client_keycode, keyname, modifiers, keycode, level)
+                keycode = self.keycode_translation.get(keyname, -1)
+                log("get_keycode(%s, %s)=%i (keyname translation)", client_keycode, keyname, keycode)
         return keycode
 
 
