@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,7 +8,7 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000041.html
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
-import subprocess
+from subprocess import Popen, PIPE, call
 import os.path
 
 from xpra.scripts.config import InitException, get_Xdummy_confdir
@@ -23,10 +23,13 @@ from xpra.platform.displayfd import read_displayfd, parse_displayfd
 
 
 VFB_WAIT = envint("XPRA_VFB_WAIT", 3)
-DEFAULT_VFB_RESOLUTION = tuple(int(x) for x in os.environ.get("XPRA_DEFAULT_VFB_RESOLUTION", "8192x4096").replace(",", "x").split("x", 1))
-assert len(DEFAULT_VFB_RESOLUTION)==2
-DEFAULT_DESKTOP_VFB_RESOLUTION = tuple(int(x) for x in os.environ.get("XPRA_DEFAULT_DESKTOP_VFB_RESOLUTION", "1280x1024").replace(",", "x").split("x", 1))
-assert len(DEFAULT_DESKTOP_VFB_RESOLUTION)==2
+def parse_resolution(envkey="XPRA_DEFAULT_VFB_RESOLUTION", default_res="8192x4096"):
+    s = os.environ.get(envkey, default_res)
+    res = tuple(int(x) for x in s.replace(",", "x").split("x", 1))
+    assert len(res)==2, "invalid resolution string '%s'" % s
+    return res
+DEFAULT_VFB_RESOLUTION = parse_resolution()
+DEFAULT_DESKTOP_VFB_RESOLUTION = parse_resolution("XPRA_DEFAULT_DESKTOP_VFB_RESOLUTION", "1280x1024")
 PRIVATE_XAUTH = envbool("XPRA_PRIVATE_XAUTH", False)
 XAUTH_PER_DISPLAY = envbool("XPRA_XAUTH_PER_DISPLAY", True)
 
@@ -42,7 +45,7 @@ def get_vfb_logger():
 def osclose(fd):
     try:
         os.close(fd)
-    except:
+    except (OSError, IOError):
         pass
 
 def create_xorg_device_configs(xorg_conf_dir, device_uuid, uid, gid):
@@ -148,7 +151,7 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
     if not os.path.exists(xauthority):
         log("creating XAUTHORITY=%s with data=%s", xauthority, xauth_data)
         try:
-            with open(xauthority, 'wa') as f:
+            with open(xauthority, "a") as f:
                 if getuid()==0 and (uid!=0 or gid!=0):
                     os.fchown(f.fileno(), uid, gid)
         except Exception as e:
@@ -178,7 +181,8 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
 
     try:
         logfile_argindex = xvfb_cmd.index('-logfile')
-        assert logfile_argindex+1<len(xvfb_cmd), "invalid xvfb command string: -logfile should not be last (found at index %i)" % logfile_argindex
+        if logfile_argindex+1>=len(xvfb_cmd):
+            raise InitException("invalid xvfb command string: -logfile should not be last")
         xorg_log_file = xvfb_cmd[logfile_argindex+1]
     except ValueError:
         xorg_log_file = None
@@ -196,8 +200,8 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
                 if POSIX and uid!=getuid() or gid!=getgid():
                     try:
                         os.lchown(xorg_log_dir, uid, gid)
-                    except:
-                        pass
+                    except (OSError, IOError):
+                        log("lchown(%s, %i, %i)", xorg_log_dir, uid, gid, exc_info=True)
             except OSError as e:
                 raise InitException("failed to create the Xorg log directory '%s': %s" % (xorg_log_dir, e))
 
@@ -210,7 +214,8 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
             log.warn("Warning: cannot use uinput")
             log.warn(" '-config' argument not found in the xvfb command")
         else:
-            assert config_argindex+1<len(xvfb_cmd), "invalid xvfb command string: -config should not be last (found at index %i)" % config_argindex
+            if config_argindex+1>=len(xvfb_cmd):
+                raise InitException("invalid xvfb command string: -config should not be last")
             xorg_conf = xvfb_cmd[config_argindex+1]
             if xorg_conf.endswith("xorg.conf"):
                 xorg_conf = xorg_conf.replace("xorg.conf", "xorg-uinput.conf")
@@ -242,8 +247,8 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
                         setuidgid(uid, gid)
                     close_fds([0, 1, 2, r_pipe, w_pipe])
                 try:
-                    xvfb = subprocess.Popen(xvfb_cmd, executable=xvfb_executable, close_fds=False,
-                                            stdin=subprocess.PIPE, preexec_fn=preexec, cwd=cwd)
+                    xvfb = Popen(xvfb_cmd, executable=xvfb_executable, close_fds=False,
+                                            stdin=PIPE, preexec_fn=preexec, cwd=cwd)
                 except OSError as e:
                     log("Popen%s", (xvfb_cmd, xvfb_executable, cwd), exc_info=True)
                     raise InitException("failed to execute xvfb command %s: %s" % (xvfb_cmd, e))
@@ -260,7 +265,7 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
             n = parse_displayfd(buf, displayfd_err)
             new_display_name = ":%s" % n
             log("Using display number provided by %s: %s", xvfb_executable, new_display_name)
-            if tmp_xorg_log_file != None:
+            if tmp_xorg_log_file:
                 #ie: ${HOME}/.xpra/Xorg.${DISPLAY}.log -> /home/antoine/.xpra/Xorg.S14700.log
                 f0 = shellsub(tmp_xorg_log_file, subs)
                 subs["DISPLAY"] = new_display_name
@@ -284,8 +289,8 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
                 else:
                     setsid()
             log("xvfb_cmd=%s", xvfb_cmd)
-            xvfb = subprocess.Popen(xvfb_cmd, executable=xvfb_executable, close_fds=True,
-                                    stdin=subprocess.PIPE, preexec_fn=preexec)
+            xvfb = Popen(xvfb_cmd, executable=xvfb_executable, close_fds=True,
+                         stdin=PIPE, preexec_fn=preexec)
 
         xauth_add(xauthority, display_name, xauth_data, uid, gid)
     except Exception as e:
@@ -298,23 +303,30 @@ def start_Xvfb(xvfb_str, pixel_depth, display_name, cwd, uid, gid, username, xau
     return xvfb, display_name, cleanups
 
 
-def set_initial_resolution(desktop=False):
+def kill_xvfb(xvfb_pid):
+    log = get_vfb_logger()
+    log.info("killing xvfb with pid %s", xvfb_pid)
+    import signal
+    try:
+        os.kill(xvfb_pid, signal.SIGTERM)
+    except OSError as e:
+        log.info("failed to kill xvfb process with pid %s:", xvfb_pid)
+        log.info(" %s", e)
+    xauthority = os.environ.get("XAUTHORITY")
+    if PRIVATE_XAUTH and xauthority and os.path.exists(xauthority):
+        os.unlink(xauthority)
+
+
+def set_initial_resolution(res=DEFAULT_VFB_RESOLUTION):
     try:
         log = get_vfb_logger()
-        log("set_initial_resolution")
-        if desktop:
-            res = DEFAULT_DESKTOP_VFB_RESOLUTION
-        else:
-            res = DEFAULT_VFB_RESOLUTION
+        log("set_initial_resolution(%s)", res)
         from xpra.x11.bindings.randr_bindings import RandRBindings      #@UnresolvedImport
         #try to set a reasonable display size:
         randr = RandRBindings()
         if not randr.has_randr():
-            l = log
-            if desktop:
-                l = log.warn
-            l("Warning: no RandR support,")
-            l(" default virtual display size unchanged")
+            log.warn("Warning: no RandR support,")
+            log.warn(" default virtual display size unchanged")
             return
         sizes = randr.get_xrr_screen_sizes()
         size = randr.get_screen_size()
@@ -323,7 +335,7 @@ def set_initial_resolution(desktop=False):
             log("RandR setting new screen size to %s", res)
             randr.set_screen_size(*res)
     except Exception as e:
-        log("set_initial_resolution(%s)", desktop, exc_info=True)
+        log("set_initial_resolution(%s)", res, exc_info=True)
         log.error("Error: failed to set the default screen size:")
         log.error(" %s", e)
 
@@ -337,7 +349,7 @@ def xauth_add(filename, display_name, xauth_data, uid, gid):
                 setuidgid(uid, gid)
         xauth_cmd = ["xauth"]+xauth_args
         start = monotonic_time()
-        code = subprocess.call(xauth_cmd, preexec_fn=preexec, close_fds=True)
+        code = call(xauth_cmd, preexec_fn=preexec, close_fds=True)
         end = monotonic_time()
         if code!=0 and (end-start>=10):
             log = get_vfb_logger()
@@ -347,7 +359,7 @@ def xauth_add(filename, display_name, xauth_data, uid, gid):
             if glob.glob("%s-*" % filename):
                 log.warn("Warning: trying to clean some stale xauth locks")
                 xauth_cmd = ["xauth", "-b"]+xauth_args
-                code = subprocess.call(xauth_cmd, preexec_fn=preexec, close_fds=True)
+                code = call(xauth_cmd, preexec_fn=preexec, close_fds=True)
         if code!=0:
             raise OSError("non-zero exit code: %s" % code)
     except OSError as e:
